@@ -19,65 +19,82 @@
     Alexander Pipelka
     pipelka@teleweb.at
  
-    Last Update:      $Author: pipelka $
-    Update Date:      $Date: 2002/03/27 10:57:02 $
-    Source File:      $Source: /usr/local/tigris/data/helm/cvs/repository/paragui/paragui/src/core/pgapplication.cpp,v $
-    CVS/RCS Revision: $Revision: 1.1.4.3 $
+    Last Update:      $Author: braindead $
+    Update Date:      $Date: 2004/06/27 17:01:31 $
+    Source File:      $Source: /cvsroot/paragui/paragui/src/core/pgapplication.cpp,v $
+    CVS/RCS Revision: $Revision: 1.2.4.22.2.18 $
     Status:           $State: Exp $
 */
 
-#include <iostream>
-#include <cstring>
-#include <cassert>
-#include <cmath>
-
 #include "pgapplication.h"
 #include "pgwidget.h"
 #include "pglayout.h"
 #include "pglog.h"
 #include "pgdraw.h"
+#include "pgtheme.h"
+
+#include <iostream>
+#include <cstring>
+#include <cassert>
+#include <cmath>
 
 // usually PARAGUI_THEMEDIR is defined by the configure script
 // or passed to the compiler. This is just a kind of last resort.
 
 #ifndef PARAGUI_THEMEDIR
-#define PARAGUI_THEMEDIR	"./"
-#endif	// PARAGUI_THEMEDIR
-
+#ifdef __MACOS__
+#define PARAGUI_THEMEDIR        ""
+#else
+#define PARAGUI_THEMEDIR        "./"
+#endif  // macintosh
+#endif  // PARAGUI_THEMEDIR
 
 SDL_mutex* PG_Application::mutexScreen = NULL;
 PG_Application* PG_Application::pGlobalApp = NULL;
 SDL_Surface* PG_Application::screen = NULL;
-std::string PG_Application::app_path = "";
+//std::string PG_Application::app_path = "";
 PG_Theme* PG_Application::my_Theme = NULL;
 bool PG_Application::bulkMode = false;
-bool PG_Application::glMode = false;
-bool PG_Application::emergencyQuit = false;
+//bool PG_Application::glMode = false;
+//bool PG_Application::emergencyQuit = false;
 bool PG_Application::enableBackground = true;
-bool PG_Application::enableAppIdleCalls = false;
-SDL_Surface *PG_Application::my_mouse_pointer;
+//bool PG_Application::enableAppIdleCalls = false;
+SDL_Surface *PG_Application::my_mouse_pointer = NULL;
+SDL_Surface *PG_Application::my_mouse_backingstore = NULL;
 PG_Rect PG_Application::my_mouse_position = PG_Rect(0,0,0,0);
-PG_CURSOR_MODE PG_Application::my_mouse_mode = PG_CURSOR_HARDWARE;
+PG_Application::CursorMode PG_Application::my_mouse_mode = PG_Application::HARDWARE;
 PG_Font* PG_Application::DefaultFont = NULL;
-SDL_Surface* PG_Application::my_background;
-SDL_Surface* PG_Application::my_scaled_background;
-SDL_Color PG_Application::my_backcolor;
-int PG_Application::my_backmode;
+SDL_Surface* PG_Application::my_background = NULL;
+SDL_Surface* PG_Application::my_scaled_background = NULL;
+PG_Color PG_Application::my_backcolor;
+int PG_Application::my_backmode = BKMODE_TILE;
+bool PG_Application::disableDirtyUpdates = false;
+//bool PG_Application::my_quitEventLoop = false;
 
 /**
 	new shutdown procedure (called at application termination
 */
 void PARAGUI_ShutDownCode() {
+
 	// shutdown SDL
 	SDL_Quit();
-
-	// shutdown PhysFS
-	PG_Application::Deinit();
 }
 
 
 PG_Application::PG_Application() {
 
+	// set UTF8 encoding if UNICODE support is enabled
+	// we use the "C" locale because it's hard to get the current locale setting
+	// in a portableway (Win32 doesn't support nl_langinfo).
+	// The "C" locale is only set for character encoding.
+#ifdef ENABLE_UNICODE
+	setlocale(LC_CTYPE, "C.UTF-8");
+#endif
+
+	my_quitEventLoop = false;
+	emergencyQuit = false;
+	enableAppIdleCalls = false;
+
 	if(pGlobalApp != NULL) {
 		PG_LogWRN("PG_Application Object already exists !");
 		exit(-1);
@@ -90,17 +107,19 @@ PG_Application::PG_Application() {
 /* Pete Shinners, Feb 1, 2001                               */
 
 #ifdef WIN32
+#ifndef __GNUC__
 	SDL_RegisterApp("ParaGUI", 0, GetModuleHandle(NULL));
+#endif
 #endif
 
 	// -- see above
 
 	/* Initialize the SDL library */
 	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE) < 0) {
-		PG_LogERR("Could not initialize SDL: %s", SDL_GetError());
+		std::cerr << "Could not initialize SDL: " << SDL_GetError() << std::endl;
 		exit(-1);
 	}
-	atexit(SDL_Quit);
+
 	pGlobalApp = this;
 	screen = NULL;
 
@@ -108,11 +127,22 @@ PG_Application::PG_Application() {
 	my_background = NULL;
 	my_freeBackground = false;
 	my_backmode = BKMODE_TILE;
+	
+	// add our base dir to the searchpath
+	AddArchive(GetBaseDir());
 }
 
 PG_Application::~PG_Application() {
-	pGlobalApp = NULL;
+	// shutdown log (before deleting all the widgets)
+	PG_LogConsole::Done();
+	
+	// remove remaining widgets
 	Shutdown();
+	
+	pGlobalApp = NULL;
+
+	// remove all archives from PG_FileArchive
+	PG_FileArchive::RemoveAllArchives();
 }
 
 /**  */
@@ -150,50 +180,48 @@ bool PG_Application::InitScreen(int w, int h, int depth, Uint32 flags) {
 }
 
 /**  */
-SDL_Thread* PG_Application::Run(bool threaded) {
-#ifndef WIN32
-	if(threaded) {
-		SDL_Thread* thrd = SDL_CreateThread(PG_Application::RunEventLoop, this);
-		return thrd;
-	}
-#endif
-	RunEventLoop(this);
-	return NULL;
+void PG_Application::Run() {
+	RunEventLoop();
 }
 
 void PG_Application::EnableAppIdleCalls(bool enable) {
 	enableAppIdleCalls = enable;
 }
 
+bool PG_Application::GetAppIdleCallsEnabled()
+{
+	return enableAppIdleCalls;
+}
+
 /** Event processing loop */
 
-int PG_Application::RunEventLoop(void* data) {
-	PG_Application* object = static_cast<PG_Application*>(data);
+void PG_Application::RunEventLoop() {
 	SDL_Event event;
 	my_quitEventLoop = false;
 
-	assert(data);
-
+	FlushEventQueue();
+	
 	while(!my_quitEventLoop) {
 		
 		// pull motion events (may flood the eventqueue)
 		while(SDL_PeepEvents(&event, 1, SDL_GETEVENT, SDL_MOUSEMOTIONMASK) > 0)
 			;
 		
+		ClearOldMousePosition();
+
 		if(enableAppIdleCalls) {
 			if (SDL_PollEvent(&event) == 0) {
-				object->eventIdle();
+				eventIdle();
 			} else {
-				PG_MessageObject::PumpIntoEventQueue(&event);
+				PumpIntoEventQueue(&event);
 			}
 		} else {
 			SDL_WaitEvent(&event);
-			PG_MessageObject::PumpIntoEventQueue(&event);
+			PumpIntoEventQueue(&event);
 		}
 
 		DrawCursor();
 	}
-	return -1;
 }
 
 
@@ -202,48 +230,107 @@ void PG_Application::ClearOldMousePosition() {
 		return;
 	}
 
-	PG_RectList *widgetList = PG_Widget::GetWidgetList();
-	PG_Application::RedrawBackground(my_mouse_position);
+	if(GetBulkMode() || my_mouse_backingstore==NULL) {
+		return;
+	}
 
-	SDL_SetClipRect(screen, &my_mouse_position);
-	PG_RectList backlist = widgetList->Intersect(&my_mouse_position);
-	backlist.Blit(my_mouse_position);
-	SDL_SetClipRect(screen, 0);
+	SDL_BlitSurface(my_mouse_backingstore, NULL, GetScreen(), &my_mouse_position);
+    
 	return;
 }
 
-void PG_Application::DrawCursor() {
+void PG_Application::DrawCursor(bool update) {
 	int x, y;
-	PG_Rect saved;
-	if(!my_mouse_pointer || my_mouse_mode != PG_CURSOR_SOFTWARE) {
+
+	if(!my_mouse_pointer || my_mouse_mode != SOFTWARE) {
 		return;
 	}
 	if(SDL_ShowCursor(SDL_QUERY) == SDL_ENABLE) {
 		// Hide hardware cursor if visible
 		SDL_ShowCursor(SDL_DISABLE);
 	}
-	//SDL_PumpEvents();
+
 	SDL_GetMouseState(&x, &y);
-	saved = my_mouse_position;
-	if(!GetBulkMode()) {
-		ClearOldMousePosition();
+
+	Sint16 dx = x - my_mouse_position.my_xpos;
+	Sint16 dy = y - my_mouse_position.my_ypos;
+
+	// calculate vertical update rect
+	PG_Rect vertical(0, my_mouse_position.my_ypos, abs(dx), my_mouse_pointer->h + abs(dy));
+	if(dx >= 0) {
+		vertical.my_xpos = my_mouse_position.my_xpos;
+	} else {
+		vertical.my_xpos = my_mouse_position.my_xpos + my_mouse_pointer->w + dx;
+	}	
+
+	// calculate horizontal update rect
+	PG_Rect horizontal(my_mouse_position.my_xpos, 0, my_mouse_pointer->w + abs(dx), abs(dy));	
+	if(dy >= 0) {
+		horizontal.my_ypos = my_mouse_position.my_ypos;
+	} else {
+		horizontal.my_ypos = my_mouse_position.my_ypos + my_mouse_pointer->h + dy;
 	}
+	
+	// clipping
+	if(vertical.my_xpos + vertical.my_width > screen->w) {
+		if(vertical.my_xpos >= screen->w) {
+			vertical.my_xpos = screen->w - 1;
+		}
+		vertical.my_width = screen->w - vertical.my_xpos;
+	}
+	if(vertical.my_ypos + vertical.my_height > screen->h) {
+		if(vertical.my_ypos >= screen->h) {
+			vertical.my_ypos = screen->h - 1;
+		}
+		vertical.my_height = screen->h - vertical.my_ypos;
+	}
+
+	if(horizontal.my_xpos + horizontal.my_width > screen->w) {
+		if(horizontal.my_xpos >= screen->w) {
+			horizontal.my_xpos = screen->w - 1;
+		}
+		horizontal.my_width = screen->w - horizontal.my_xpos;
+	}
+	if(horizontal.my_ypos + horizontal.my_height > screen->h) {
+		if(horizontal.my_ypos >= screen->h) {
+			horizontal.my_ypos = screen->h - 1;
+		}
+		horizontal.my_height = screen->h - horizontal.my_ypos;
+	}		
+
 	my_mouse_position.my_xpos = x;
 	my_mouse_position.my_ypos = y;
+	my_mouse_position.my_width = my_mouse_pointer->w;
+	my_mouse_position.my_height = my_mouse_pointer->h;
+	
+	// backup current cursor area
+	if(my_mouse_backingstore == NULL) {
+		my_mouse_backingstore = PG_Draw::CreateRGBSurface(my_mouse_pointer->w, my_mouse_pointer->h);
+	}
+	SDL_BlitSurface(GetScreen(), &my_mouse_position, my_mouse_backingstore, NULL);
+	
+	// draw cursor
 	SDL_BlitSurface(my_mouse_pointer, 0, screen, &my_mouse_position);
-	if(!GetBulkMode()) {
-		SDL_UpdateRects(screen, 1, &saved);
-		SDL_UpdateRects(screen, 1, &my_mouse_position);
+
+	if(!GetBulkMode() && update) {
+		SDL_Rect rects[3] = {horizontal, vertical, my_mouse_position};
+		SDL_UpdateRects(screen, 3, rects);
 	}
 }
 void PG_Application::Quit() {
-	SendMessage(this, MSG_QUIT, PG_IDAPPLICATION, 0);
+	sigQuit(this);
+	eventQuit(0, this, 0);
 }
 
 /**  */
 bool PG_Application::eventKeyDown(const SDL_KeyboardEvent* key) {
-
-	if (key->keysym.sym == PG_LOGCONSOLE_KEY) {
+	SDLKey ckey = PG_LogConsole::GetConsoleKey();
+	
+	if(ckey == 0) {
+		return false;
+	}
+	
+	if (key->keysym.sym == ckey) {
 		PG_LogConsole::Update();
 		PG_LogConsole::Toggle();
 		return true;
@@ -255,7 +342,6 @@ bool PG_Application::eventKeyDown(const SDL_KeyboardEvent* key) {
 bool PG_Application::eventKeyUp(const SDL_KeyboardEvent* key) {
 
 	if((key->keysym.sym == SDLK_ESCAPE) && emergencyQuit) {
-
 		Quit();
 		return true;
 	}
@@ -274,7 +360,7 @@ bool PG_Application::eventResize(const SDL_ResizeEvent* event) {
 
 	PG_Widget::UpdateRect(PG_Rect(0,0,event->w,event->h));
 	SDL_UpdateRect(screen,0,0,event->w,event->h);
-	SendMessage(NULL, MSG_VIDEORESIZE, 0, 0);
+	sigVideoResize(this, event);
 
 	return true;
 }
@@ -284,8 +370,10 @@ void PG_Application::SetCursor(SDL_Surface *image) {
 		if(!my_mouse_pointer) {
 			return;
 		}
+		UnloadSurface(my_mouse_backingstore);
+		my_mouse_backingstore = NULL;
 		UnloadSurface(my_mouse_pointer);
-		my_mouse_pointer = 0;
+		my_mouse_pointer = NULL;
 		ClearOldMousePosition();
 		SDL_UpdateRects(screen, 1, &my_mouse_position);
 		SDL_ShowCursor(SDL_ENABLE);
@@ -297,27 +385,30 @@ void PG_Application::SetCursor(SDL_Surface *image) {
 		UnloadSurface(my_mouse_pointer);
 		my_mouse_pointer = image;
 	}
+
+	UnloadSurface(my_mouse_backingstore);
+	my_mouse_backingstore = NULL;
 	image->refcount++;
 	DrawCursor();
 }
 
-PG_CURSOR_MODE PG_Application::ShowCursor(PG_CURSOR_MODE mode) {
+PG_Application::CursorMode PG_Application::ShowCursor(CursorMode mode) {
 	switch(mode) {
-		case PG_CURSOR_NONE:
+		case NONE:
 			SDL_ShowCursor(SDL_DISABLE);
 			break;
-		case PG_CURSOR_HARDWARE:
+		case HARDWARE:
 			SDL_ShowCursor(SDL_ENABLE);
 			break;
-		case PG_CURSOR_SOFTWARE:
+		case SOFTWARE:
 			DrawCursor();
 			break;
-		case PG_CURSOR_QUERY:
+		case QUERY:
 			return my_mouse_mode;
 
 	}
-	PG_CURSOR_MODE orig = my_mouse_mode;
-	if(mode != PG_CURSOR_SOFTWARE && my_mouse_mode == PG_CURSOR_SOFTWARE) {
+	CursorMode orig = my_mouse_mode;
+	if(mode != SOFTWARE && my_mouse_mode == SOFTWARE) {
 		ClearOldMousePosition();
 		SDL_UpdateRects(screen, 1, &my_mouse_position);
 	}
@@ -333,7 +424,7 @@ SDL_Surface* PG_Application::SetScreen(SDL_Surface* surf) {
         
 	PG_Application::screen = surf;
 
-	glMode = (surf->flags & SDL_OPENGLBLIT);
+	//glMode = (surf->flags & SDL_OPENGLBLIT);
 
 	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
 	SDL_EnableUNICODE(true);
@@ -349,6 +440,11 @@ bool PG_Application::SetBackground(const char* filename, int mode) {
         if (!filename)
                 return false;
         
+	if(my_freeBackground && my_background) {
+		UnloadSurface(my_background);
+		my_freeBackground = false;
+	}
+		
 	my_background = LoadSurface(filename);
 
 	if(my_scaled_background) {
@@ -360,6 +456,7 @@ bool PG_Application::SetBackground(const char* filename, int mode) {
 		my_freeBackground = true;
 		my_backmode = mode;
 		RedrawBackground(PG_Rect(0,0,screen->w,screen->h));
+		PG_Widget::GetWidgetList()->Blit();
 		return true;
 	} else {
 		PG_LogWRN("Failed to load '%s'",(char*)filename);
@@ -387,24 +484,20 @@ bool PG_Application::SetBackground(SDL_Surface* surface, int mode) {
 	my_backmode = mode;
 
 	RedrawBackground(PG_Rect(0,0,screen->w,screen->h));
+	PG_Widget::GetWidgetList()->Blit();
 	return true;
 }
 
 /**  */
 void PG_Application::RedrawBackground(const PG_Rect& rect) {
-	static PG_Rect screenrect(0,0,screen->w,screen->h);
-
-	if(GetBulkMode() || !enableBackground) {
+	if(GetBulkMode()) {
 		return;
 	}
 
 	PG_Rect fillrect = rect;
-	PG_Application::LockScreen();
 
-	if(!my_background) {
-		SDL_Color c = my_backcolor;
-		SDL_FillRect(screen, (SDL_Rect*)&fillrect, SDL_MapRGB(screen->format, c.r, c.g, c.b));
-		PG_Application::UnlockScreen();
+	if(!my_background || !enableBackground) {
+		SDL_FillRect(screen, (SDL_Rect*)&fillrect, my_backcolor.MapRGB(screen->format));
 		return;
 	}
 	if(my_backmode == BKMODE_STRETCH &&
@@ -413,35 +506,37 @@ void PG_Application::RedrawBackground(const PG_Rect& rect) {
 		if(my_scaled_background && 
 		   (my_scaled_background->w != screen->w ||
 		    my_scaled_background->h != screen->h)) {
-			SDL_FreeSurface(my_scaled_background); // size mismatch
-			my_scaled_background = 0;
+			UnloadSurface(my_scaled_background); // size mismatch
+			my_scaled_background = NULL;
 		}
 		if(!my_scaled_background) {
-			my_scaled_background =
-				PG_Draw::ScaleSurface(my_background,
-						      static_cast<Uint16>(screen->w), static_cast<Uint16>(screen->h));
+			SDL_Surface* temp = PG_Draw::ScaleSurface(my_background, static_cast<Uint16>(screen->w), static_cast<Uint16>(screen->h));
+			my_scaled_background = SDL_DisplayFormat(temp);
+			UnloadSurface(temp);
+			/*PG_Draw::ScaleSurface(my_background,
+						      static_cast<Uint16>(screen->w), static_cast<Uint16>(screen->h));*/
 		}
 		SDL_GetClipRect(screen, (SDL_Rect*)&fillrect);
 		SDL_SetClipRect(screen, (SDL_Rect*)&rect);
-		SDL_BlitSurface(my_scaled_background, 0, screen, 0);
+		SDL_SetAlpha(my_scaled_background, 0, 0);
+		SDL_BlitSurface(my_scaled_background, (SDL_Rect*)&rect, screen, (SDL_Rect*)&rect);
 		SDL_SetClipRect(screen, (SDL_Rect*)&fillrect);
 
 	} else {
-		PG_Draw::DrawTile(screen, screenrect, rect, my_background);
+		SDL_SetAlpha(my_background, 0, 0);
+		PG_Draw::DrawTile(screen, PG_Rect(0,0,screen->w,screen->h), rect, my_background);
 	}
-	
-	PG_Application::UnlockScreen();
 }
 
 /**  */
-const char* PG_Application::GetApplicationPath() {
+/*const char* PG_Application::GetApplicationPath() {
 	return app_path.c_str();
-}
+}*/
 
-void PG_Application::SetApplicationPath(const char* path) {
+/*void PG_Application::SetApplicationPath(const char* path) {
 	AddArchive(path);
 	app_path = path;
-}
+}*/
 
 /**  */
 const char* PG_Application::GetRelativePath(const char* file) {
@@ -524,22 +619,36 @@ PG_Theme* PG_Application::LoadTheme(const char* xmltheme, bool asDefault, const
 
 	PG_LogDBG("Locating theme '%s' ...", xmltheme);
 
-	// MacOS does not use file path separator '/', instead ':' is used
-	// There could be clever solution for this, but for a while...
-	// let's assume that "data" directory must exist in working directory on MacOS.
-	// Masahiro Minami<elsur@aaa.letter.co.jp>
-	// 01/05/06
-    
-	// add paths to the archive
-
-	//#ifndef macintosh
-
 	if(searchpath != NULL) {
 		if(AddArchive(searchpath)) {
 			PG_LogDBG("'%s' added to searchpath", searchpath);
 		}
 	}
 
+#ifdef __MACOS__
+
+	if(AddArchive("")) {
+		PG_LogDBG("'' added to searchpath");
+	}
+
+	if(AddArchive(":")) {
+		PG_LogDBG("':' added to searchpath");
+	}
+
+	if(AddArchive(":data:")) {
+		PG_LogDBG("':data:' added to searchpath");
+	}
+
+	if(AddArchive("::data:")) {
+		PG_LogDBG("'::data:' added to searchpath");
+	}
+
+	if(PARAGUI_THEMEDIR != NULL) {
+		PG_LogDBG("'"PARAGUI_THEMEDIR"' added to searchpath");
+	}
+
+#else	
+
 	if(AddArchive("./")) {
 		PG_LogDBG("'./' added to searchpath");
 	}
@@ -562,6 +671,8 @@ PG_Theme* PG_Application::LoadTheme(const char* xmltheme, bool asDefault, const
 		}
 	}
 
+#endif // __MACOS__
+
 	if(AddArchive(PARAGUI_THEMEDIR)) {
 		PG_LogDBG("'"PARAGUI_THEMEDIR "' added to searchpath");
 	}
@@ -584,11 +695,9 @@ PG_Theme* PG_Application::LoadTheme(const char* xmltheme, bool asDefault, const
 		PG_LogMSG("size: %i", DefaultFont->GetSize());
 
 		my_background = theme->FindSurface("Background", "Background", "background");
-		my_backmode = theme->FindProperty("Background", "Background", "backmode");
-		SDL_Color* bc = theme->FindColor("Background", "Background", "backcolor");
-		if(bc != NULL) {
-			my_backcolor = *bc;
-		}
+		theme->GetProperty("Background", "Background", "backmode", my_backmode);
+		theme->GetColor("Background", "Background", "backcolor", my_backcolor);
+
 		if(my_scaled_background) {
 			// Destroyed scaled background if present
 			SDL_FreeSurface(my_scaled_background);
@@ -632,26 +741,19 @@ void PG_Application::SetBulkMode(bool bulk) {
 }
 
 void PG_Application::Shutdown() {
-	if(my_scaled_background) {
-		// Destroyed scaled background if present
-		SDL_FreeSurface(my_scaled_background);
-		my_scaled_background = 0;
-	}
+	DeleteBackground();
+	
 	// destroy still existing objects
-	std::vector<PG_MessageObject*>::iterator list = objectList.begin();
+	//vector<PG_MessageObject*>::iterator list = objectList.begin();
+	PG_Widget* list = PG_Widget::GetWidgetList()->first();
 
-	while(list != objectList.end()) {
+	while(list != NULL) {
 
-		if(*list == this) {
-			list++;
-			continue;
-		}
+		PG_Widget* o = list;
+		list = list->next();
 
-		PG_MessageObject* temp = *list;
-		//RemoveObject(*list);
-		delete temp;
-
-		list = objectList.begin();
+		PG_Widget::GetWidgetList()->Remove(o);
+		delete o;
 	}
 
 	// unload theme (and free the allocated mem)
@@ -660,18 +762,16 @@ void PG_Application::Shutdown() {
 		my_Theme = NULL;
 	}
 
-	// shutdown log
-	PG_LogConsole::Done();
-
-	// remove all archives from PG_FileArchive
-	RemoveAllArchives();
-
 	// destroy screen mutex
 	SDL_DestroyMutex(mutexScreen);
 
 	// delete the default font
 	delete DefaultFont;
 	DefaultFont = NULL;
+
+	// remove cursor backing store	
+	UnloadSurface(my_mouse_backingstore);
+	my_mouse_backingstore = NULL;
 }
 
 void PG_Application::SetEmergencyQuit(bool esc) {
@@ -738,17 +838,17 @@ void PG_Application::SetIcon(const char *filename) {
 
 	//Clean up
 	delete[] mask;
-	return;
 }
 
 void PG_Application::SetCaption(const char *title, const char *icon) {
-	SDL_WM_SetCaption(title,icon);
-	return;
+	SDL_WM_SetCaption(title, NULL);
+	if (icon != NULL) {
+		SetIcon(icon);
+	}
 }
 
 void PG_Application::GetCaption(char **title, char **icon) {
 	SDL_WM_GetCaption(title,icon);
-	return;
 }
 
 int PG_Application::Iconify(void) {
@@ -778,24 +878,24 @@ static PG_Widget *FindInChildObjects(PG_RectList *RectList, const char *Name) {
 	if (!Name)
             return NULL;
     
-	PG_RectList::iterator list = RectList->begin();
+	PG_Widget* list = RectList->first();
 
-	while(list != RectList->end()) {
-		if (strcmp((*list)->GetName(), Name) == 0) {
-			return((*list));
+	while(list != NULL) {
+		if (strcmp(list->GetName(), Name) == 0) {
+			return list;
 		}
 
-		PG_Widget* result = (*list)->FindChild(Name);
+		PG_Widget* result = list->FindChild(Name);
 		if(result != NULL) {
 			return result;
 		}
 
-		retWidget = FindInChildObjects((*list)->GetChildList(), Name);
+		retWidget = FindInChildObjects(list->GetChildList(), Name);
 		if (retWidget != NULL) {
 			return retWidget;
 		}
 
-		list++;
+		list = list->next();
 	}
 
 	return NULL;
@@ -811,25 +911,25 @@ static inline PG_Widget *FindInChildObjects(PG_RectList *RectList, int id) {
 	if (id < 0)
             return 0;
     
-	PG_RectList::iterator list = RectList->begin();
+	PG_Widget* list = RectList->first();
 
-	while(list != RectList->end()) {
-		if ((*list)->GetID() == id) {
-			return((*list));
+	while(list != NULL) {
+		if (list->GetID() == id) {
+			return list;
 		}
 
 
-		PG_Widget* result = (*list)->FindChild(id);
+		PG_Widget* result = list->FindChild(id);
 		if(result != NULL) {
 			return result;
 		}
         
-		retWidget = FindInChildObjects((*list)->GetChildList(), id);
+		retWidget = FindInChildObjects(list->GetChildList(), id);
 		if (retWidget != NULL) {
 			return retWidget;
 		}
 
-		list++;
+		list = list->next();
 	}
 
 	return NULL;
@@ -843,31 +943,15 @@ PG_Widget *PG_Application::GetWidgetById(int id) {
         return (FindInChildObjects(PG_Widget::GetWidgetList(), id));
 }
 
-void PG_Application::SetFontColor(const SDL_Color& Color) {
+void PG_Application::SetFontColor(const PG_Color& Color) {
 	DefaultFont->SetColor(Color);
 }
 
-void PG_Application::SetFontColor(int Red, int Green, int Blue) {
-	SDL_Color Color;
-	Color.r = Red;
-	Color.g = Green;
-	Color.b = Blue;
-	SetFontColor(Color);
-}
-
-void PG_Application::SetFontColor(int c) {
-	SDL_Color Color;
-	Color.r = (c >> 16) & 0xff;
-	Color.g = (c >> 8) & 0xff;
-	Color.b = c & 0xff;
-	SetFontColor(Color);
-}
-
 void PG_Application::SetFontAlpha(int Alpha) {
 	DefaultFont->SetAlpha(Alpha);
 }
 
-void PG_Application::SetFontStyle(int Style) {
+void PG_Application::SetFontStyle(PG_Font::Style Style) {
 	DefaultFont->SetStyle(Style);
 }
 
@@ -883,9 +967,9 @@ void PG_Application::SetFontName(const char *Name) {
 	DefaultFont->SetName(Name);
 }
 
-SDL_Surface* PG_Application::GetScreen() {
+/*SDL_Surface* PG_Application::GetScreen() {
 	return screen;
-}
+}*/
 
 int PG_Application::GetScreenHeight() {
 	return screen->h;
@@ -903,14 +987,30 @@ bool PG_Application::GetBulkMode() {
 	return bulkMode;
 }
 
-bool PG_Application::GetGLMode() {
+/*bool PG_Application::GetGLMode() {
 		return glMode;
-}
+}*/
 
 void PG_Application::EnableBackground(bool enable) {
 	enableBackground = enable;
 }
 
+void PG_Application::DeleteBackground() {
+ 	enableBackground = false;
+
+ 	if(my_scaled_background) {
+ 		// Destroyed scaled background if present
+ 		SDL_FreeSurface(my_scaled_background);
+ 		my_scaled_background = 0;
+ 	}
+	if(my_freeBackground && my_background) {
+		UnloadSurface(my_background);
+		my_freeBackground = false;
+	}
+
+	my_background = 0;
+}
+
 #ifdef WIN32
 bool PG_Application::LockScreen() {
 	return (SDL_mutexP(mutexScreen) == 0);
@@ -923,9 +1023,271 @@ bool PG_Application::UnlockScreen() {
 }
 #endif
 
+void PG_Application::DisableDirtyUpdates(bool disable) {
+	disableDirtyUpdates = disable;
+}
+
+bool PG_Application::GetDirtyUpdatesDisabled() {
+	return disableDirtyUpdates;
+}
+
+PG_Application* PG_Application::GetApp() {
+	return pGlobalApp;
+}
+
+void PG_Application::FlushEventQueue() {
+	SDL_Event event;
+
+	while(SDL_PollEvent(&event)) {
+		/*if(event.type == SDL_USEREVENT) {
+			delete (MSG_MESSAGE*)(event.user.data1);
+		}*/
+	}
+}
+
+#ifdef WIN32
+SDL_Surface* PG_Application::GetScreen() {
+	return screen;
+}
+#endif
+
+void PG_Application::eventIdle() {
+	sigAppIdle(this);
+	SDL_Delay(1);
+}
+
+void PG_Application::TranslateNumpadKeys(SDL_KeyboardEvent *key) {
+	// note: works on WIN, test this on other platforms
+
+	// numeric keypad translation
+	if (key->keysym.unicode==0) {	 // just optimalisation
+		if (key->keysym.mod & KMOD_NUM) {
+			// numeric keypad is enabled
+			switch (key->keysym.sym) {
+				case SDLK_KP0       :
+					key->keysym.sym = SDLK_0;
+					key->keysym.unicode = SDLK_0;
+					break;
+				case SDLK_KP1       :
+					key->keysym.sym = SDLK_1;
+					key->keysym.unicode = SDLK_1;
+					break;
+				case SDLK_KP2       :
+					key->keysym.sym = SDLK_2;
+					key->keysym.unicode = SDLK_2;
+					break;
+				case SDLK_KP3       :
+					key->keysym.sym = SDLK_3;
+					key->keysym.unicode = SDLK_3;
+					break;
+				case SDLK_KP4       :
+					key->keysym.sym = SDLK_4;
+					key->keysym.unicode = SDLK_4;
+					break;
+				case SDLK_KP5       :
+					key->keysym.sym = SDLK_5;
+					key->keysym.unicode = SDLK_5;
+					break;
+				case SDLK_KP6       :
+					key->keysym.sym = SDLK_6;
+					key->keysym.unicode = SDLK_6;
+					break;
+				case SDLK_KP7       :
+					key->keysym.sym = SDLK_7;
+					key->keysym.unicode = SDLK_7;
+					break;
+				case SDLK_KP8       :
+					key->keysym.sym = SDLK_8;
+					key->keysym.unicode = SDLK_8;
+					break;
+				case SDLK_KP9       :
+					key->keysym.sym = SDLK_9;
+					key->keysym.unicode = SDLK_9;
+					break;
+				case SDLK_KP_PERIOD :
+					key->keysym.sym = SDLK_PERIOD;
+					key->keysym.unicode = SDLK_PERIOD;
+					break;
+				case SDLK_KP_DIVIDE :
+					key->keysym.sym = SDLK_BACKSLASH;
+					key->keysym.unicode = SDLK_BACKSLASH;
+					break;
+				case SDLK_KP_MULTIPLY:
+					key->keysym.sym = SDLK_ASTERISK;
+					key->keysym.unicode = SDLK_ASTERISK;
+					break;
+				case SDLK_KP_MINUS  :
+					key->keysym.sym = SDLK_MINUS;
+					key->keysym.unicode = SDLK_MINUS;
+					break;
+				case SDLK_KP_PLUS   :
+					key->keysym.sym = SDLK_PLUS;
+					key->keysym.unicode = SDLK_PLUS;
+					break;
+				case SDLK_KP_ENTER  :
+					key->keysym.sym = SDLK_RETURN;
+					key->keysym.unicode = SDLK_RETURN;
+					break;
+				case SDLK_KP_EQUALS :
+					key->keysym.sym = SDLK_EQUALS;
+					key->keysym.unicode = SDLK_EQUALS;
+					break;
+
+				default:
+					break;
+			}
+		} else {
+			// numeric keypad is disabled
+			switch (key->keysym.sym) {
+				case SDLK_KP0       :
+					key->keysym.sym = SDLK_INSERT;
+					key->keysym.unicode = 0;
+					break;
+				case SDLK_KP1       :
+					key->keysym.sym = SDLK_END;
+					key->keysym.unicode = 0;
+					break;
+				case SDLK_KP2       :
+					key->keysym.sym = SDLK_DOWN;
+					key->keysym.unicode = 0;
+					break;
+				case SDLK_KP3       :
+					key->keysym.sym = SDLK_PAGEDOWN;
+					key->keysym.unicode = 0;
+					break;
+				case SDLK_KP4       :
+					key->keysym.sym = SDLK_LEFT;
+					key->keysym.unicode = 0;
+					break;
+				case SDLK_KP6       :
+					key->keysym.sym = SDLK_RIGHT;
+					key->keysym.unicode = 0;
+					break;
+				case SDLK_KP7       :
+					key->keysym.sym = SDLK_HOME;
+					key->keysym.unicode = 0;
+					break;
+				case SDLK_KP8       :
+					key->keysym.sym = SDLK_UP;
+					key->keysym.unicode = 0;
+					break;
+				case SDLK_KP9       :
+					key->keysym.sym = SDLK_PAGEUP;
+					key->keysym.unicode = 0;
+					break;
+				case SDLK_KP_PERIOD :
+					key->keysym.sym = SDLK_DELETE;
+					key->keysym.unicode = 0;
+					break;
+				case SDLK_KP_DIVIDE :
+					key->keysym.sym = SDLK_BACKSLASH;
+					key->keysym.unicode = SDLK_BACKSLASH;
+					break;
+				case SDLK_KP_MULTIPLY:
+					key->keysym.sym = SDLK_ASTERISK;
+					key->keysym.unicode = SDLK_ASTERISK;
+					break;
+				case SDLK_KP_MINUS  :
+					key->keysym.sym = SDLK_MINUS;
+					key->keysym.unicode = SDLK_MINUS;
+					break;
+				case SDLK_KP_PLUS   :
+					key->keysym.sym = SDLK_PLUS;
+					key->keysym.unicode = SDLK_PLUS;
+					break;
+				case SDLK_KP_ENTER  :
+					key->keysym.sym = SDLK_RETURN;
+					key->keysym.unicode = SDLK_RETURN;
+					break;
+				case SDLK_KP_EQUALS :
+					key->keysym.sym = SDLK_EQUALS;
+					key->keysym.unicode = SDLK_EQUALS;
+					break;
+
+				default:
+					break;
+			}
+		}
+	}
+}
+
+bool PG_Application::PumpIntoEventQueue(const SDL_Event* event) {
+	PG_Widget* widget = NULL;
+
+	// do we have a capture hook?
+	if((event->type != SDL_USEREVENT) && (event->type != SDL_VIDEORESIZE)) {
+		if(captureObject) {
+			return captureObject->ProcessEvent(event);
+		}
+	}
+
+	switch(event->type) {
+
+		case SDL_KEYDOWN:
+		case SDL_KEYUP:
+			if(inputFocusObject) {
+				// first send it to the focus object
+				if(inputFocusObject->ProcessEvent(event)) {
+					return true;
+				}
+				// if the focus object doesn't respond -> pump it into the queue
+			}
+			break;
+
+		case SDL_MOUSEMOTION:
+			widget = PG_Widget::FindWidgetFromPos(event->motion.x, event->motion.y);
+
+			if(lastwidget && (lastwidget != widget)) {
+				lastwidget->eventMouseLeave();
+				lastwidget = NULL;
+			}
+
+			if(widget) {
+				lastwidget = widget;
+				widget->ProcessEvent(event);
+				return true;
+			}
+			return true;
+
+		case SDL_MOUSEBUTTONUP:
+		case SDL_MOUSEBUTTONDOWN:
+			widget = PG_Widget::FindWidgetFromPos(event->button.x, event->button.y);
+			if(widget) {
+				widget->ProcessEvent(event);
+				return true;
+			}
+			break;
+	}
+
+	// first let's try to process the event ourself
+	if(ProcessEvent(event)) {
+		return true;
+	}
+
+	// send to all receivers
+	bool processed = false;
+	PG_Widget* list = PG_Widget::GetWidgetList()->first();
+	//vector<PG_MessageObject*>::iterator list = objectList.begin();
+	PG_Widget* o = NULL;
+	while(list != NULL) {
+		o = list;
+		/*if(o == NULL) {
+			list = o->next();
+			PG_Widget::GetWidgetList()->Remove(o);
+			continue;
+		}*/
+		if(o->ProcessEvent(event)) {
+			processed = true;
+			break;		// exit loop if an object responds
+		}
+		list = list->next();
+	}
+
+	return processed;
+}
+
 /*
  * Local Variables:
  * c-basic-offset: 8
  * End:
  */
-
