@@ -19,20 +19,20 @@
     Alexander Pipelka
     pipelka@teleweb.at
  
-    Last Update:      $Author: pipelka $
-    Update Date:      $Date: 2002/02/09 13:45:10 $
-    Source File:      $Source: /usr/local/tigris/data/helm/cvs/repository/paragui/paragui/src/draw/stretch.cpp,v $
-    CVS/RCS Revision: $Revision: 1.1.4.1 $
+    Last Update:      $Author: braindead $
+    Update Date:      $Date: 2004/03/13 13:46:03 $
+    Source File:      $Source: /cvsroot/paragui/paragui/src/draw/stretch.cpp,v $
+    CVS/RCS Revision: $Revision: 1.3.6.2.2.2 $
     Status:           $State: Exp $
 */
 
+#include "pgdraw.h"
+
 #include <cstdio>
 #include <cstring>
 #include <cstdlib>
 #include <cmath>
 
-#include "pgdraw.h"
-
 #if ! defined( macintosh ) && ! defined( __MWERKS__ )
 // malloc.h should not be included on Mac with CodeWarrior (pro 4)
 // Masahiro Minami<elsur@aaa.letter.co.jp>
@@ -406,12 +406,19 @@ CLIST	*contrib; 		/* array of contribution lists */
 static double filterLUT[4001];
 
 void PG_Draw::CreateFilterLUT() {
+	static bool bCreated = false;
 	double v;
 
+	if(bCreated) {
+		return;
+	}
+	
 	for (int i = -2000; i < 2001; i++) {
 		v = (double)i / 1000.0;
 		filterLUT[i + 2000] = B_spline_filter(v);
 	}
+	
+	bCreated = true;
 }
 
 typedef double (*filterfunc)(double);
