@@ -19,10 +19,10 @@
     Alexander Pipelka
     pipelka@teleweb.at
  
-    Last Update:      $Author: pipelka $
-    Update Date:      $Date: 2002/02/21 19:19:47 $
-    Source File:      $Source: /usr/local/tigris/data/helm/cvs/repository/paragui/paragui/src/font/pgfont.cpp,v $
-    CVS/RCS Revision: $Revision: 1.1.4.2 $
+    Last Update:      $Author: braindead $
+    Update Date:      $Date: 2004/09/27 09:42:34 $
+    Source File:      $Source: /cvsroot/paragui/paragui/src/font/pgfont.cpp,v $
+    CVS/RCS Revision: $Revision: 1.3.6.3.2.7 $
     Status:           $State: Exp $
 */
 
@@ -30,6 +30,7 @@
 #include "pglog.h"
 #include "pgfilearchive.h"
 #include "facecache.h"
+#include "pgstring.h"
 
 //#define OLD_TEXTRENDERING
 
@@ -93,27 +94,30 @@ void PG_FontEngine::FontEngineError(FT_Error error) {
 
 PG_GlyphCacheItem* PG_FontEngine::GetGlyph(PG_Font *Param, int glyph_index) {
 
-	PG_GlyphCacheItem *GlyphCacheItem = Param->GetFaceCache()->GlyphCache[glyph_index];
+	PG_FontFaceCacheItem* facecache = Param->GetFaceCache();
+	PG_GlyphCacheItem *GlyphCacheItem = facecache->GlyphCache[glyph_index];
 
 	if(GlyphCacheItem != NULL) {
 		return GlyphCacheItem;
 	}
 
-	if (FT_Load_Glyph(Param->GetFaceCache()->Face, glyph_index, FT_LOAD_RENDER ))
+	FT_Face face = facecache->Face;
+
+	if (FT_Load_Glyph(face, glyph_index, FT_LOAD_RENDER ))
 		return NULL;
 
-	Uint32 bitmapsize = Param->GetFaceCache()->Face->glyph->bitmap.pitch * Param->GetFaceCache()->Face->glyph->bitmap.rows;
-	GlyphCacheItem =  new PG_GlyphCacheItem(bitmapsize);
+	Uint32 bitmapsize = face->glyph->bitmap.pitch * face->glyph->bitmap.rows;
+	GlyphCacheItem = new PG_GlyphCacheItem(bitmapsize);
 
 	GlyphCacheItem->Glyph_Index = glyph_index;
-	GlyphCacheItem->Bitmap = Param->GetFaceCache()->Face->glyph->bitmap;
-	GlyphCacheItem->Bitmap_left = Param->GetFaceCache()->Face->glyph->bitmap_left;
-	GlyphCacheItem->Bitmap_top = Param->GetFaceCache()->Face->glyph->bitmap_top;
-	GlyphCacheItem->Advance_x = FT_CEIL(Param->GetFaceCache()->Face->glyph->metrics.horiAdvance);
-	memcpy(GlyphCacheItem->data(), Param->GetFaceCache()->Face->glyph->bitmap.buffer, bitmapsize);
+	GlyphCacheItem->Bitmap = face->glyph->bitmap;
+	GlyphCacheItem->Bitmap_left = face->glyph->bitmap_left;
+	GlyphCacheItem->Bitmap_top = face->glyph->bitmap_top;
+	GlyphCacheItem->Advance_x = FT_CEIL(face->glyph->metrics.horiAdvance);
+	memcpy(GlyphCacheItem->data(), face->glyph->bitmap.buffer, bitmapsize);
 	GlyphCacheItem->Bitmap.buffer = (unsigned char*)GlyphCacheItem->data();
 
-	Param->GetFaceCache()->GlyphCache[glyph_index] = GlyphCacheItem;
+	facecache->GlyphCache[glyph_index] = GlyphCacheItem;
 	return GlyphCacheItem;
 }
 
@@ -145,7 +149,7 @@ inline void BlitTemplate(DT pixels, SDL_Surface* Surface, FT_Bitmap *Bitmap, int
 	Uint32 pitch = Surface->pitch;
 	Uint32 src_pitch = Bitmap->pitch;
 	register Uint8* src_pixels = Bitmap->buffer + x0 + y0*Bitmap->pitch;
-	register Uint8* dst_pixels = (Uint8*)pixels + (PosX+x0)*bpp + (PosY+y0)*pitch;
+	register Uint8* dst_pixels = (Uint8*)pixels + (PosX+x0)*bpp + (PosY+y0)*pitch /*+ Surface->offset*/;
 	Uint8* line;
 
 	Uint8 r,g,b,a;
@@ -154,10 +158,11 @@ inline void BlitTemplate(DT pixels, SDL_Surface* Surface, FT_Bitmap *Bitmap, int
 	Uint32 color = 0;
 	Sint32 v;
 
-	SDL_Color fc = Param->GetColor();
+	PG_Color fc = Param->GetColor();
 	cr = fc.r;
 	cg = fc.g;
 	cb = fc.b;
+
 	int alpha = Param->GetAlpha();
 
 	line = dst_pixels;
@@ -181,8 +186,8 @@ inline void BlitTemplate(DT pixels, SDL_Surface* Surface, FT_Bitmap *Bitmap, int
 
 			// Get the pixel
 			color = *((DT) (dst_pixels));
-			if ( Surface->format->BytesPerPixel > 1 ) {
-
+			switch(Surface->format->BytesPerPixel) {
+			default:
 				// get the RGBA values
 				rv = (color & Rmask) >> Rshift;
 				r = (rv << Rloss) + (rv >> Rloss8);
@@ -221,11 +226,39 @@ inline void BlitTemplate(DT pixels, SDL_Surface* Surface, FT_Bitmap *Bitmap, int
 
 				// get the destination color
 				color = (r >> Rloss) << Rshift
-        		    | (g >> Gloss) << Gshift
-                	    | (b >> Bloss) << Bshift
+					| (g >> Gloss) << Gshift
+					| (b >> Bloss) << Bshift
 	        	    | ((a >> Aloss) << Ashift & Amask);
 				// Set the pixel
-			} else {
+				*((DT) (dst_pixels)) = color;
+				break;
+			
+			case 3:																								
+				cr = (fc.r << format->Rshift) >> 16 & 0xff;
+				cg = (fc.g << format->Gshift) >> 8 & 0xff;				
+				cb = fc.b << format->Bshift & 0xff;
+				
+				if (v == 255) {
+					r = cr;
+					g = cg;
+					b = cb;
+				}
+				// calculate new RGB values
+				else {
+					b = *(dst_pixels);
+					g = *(dst_pixels+1);
+					r = *(dst_pixels+2);
+					r += ((cr - r) * v) >> 8;
+					g += ((cg - g) * v) >> 8;
+					b += ((cb - b) * v) >> 8;
+				}								
+ 								
+				*dst_pixels = b;
+				*(dst_pixels + 1) = g;
+				*(dst_pixels + 2) = r;
+				break;
+			
+			case 1:
 				SDL_GetRGBA(color, format, &r, &g, &b, &a);
 
 				// calculate new RGBA values
@@ -249,8 +282,9 @@ inline void BlitTemplate(DT pixels, SDL_Surface* Surface, FT_Bitmap *Bitmap, int
 					a = v;
 				}
  				color = SDL_MapRGBA(format, r,g,b, a);
-			}
-			*((DT) (dst_pixels)) = color;
+				*((DT) (dst_pixels)) = color;
+				break;			
+			}			
 
 		}
 		src_pixels -= xw;
@@ -310,8 +344,9 @@ bool PG_FontEngine::BlitFTBitmap(SDL_Surface *Surface, FT_Bitmap *Bitmap, int Po
 		return false;
 	}
 	
-	switch(Surface->format->BytesPerPixel) {
+	switch(Surface->format->BytesPerPixel) {						
 		case 1:
+		case 3:
 			BlitTemplate((Uint8*)Surface->pixels, Surface, Bitmap, PosX, PosY, x0, x1, y0, y1, Param);
 			break;
 		case 2:
@@ -320,6 +355,9 @@ bool PG_FontEngine::BlitFTBitmap(SDL_Surface *Surface, FT_Bitmap *Bitmap, int Po
 		case 4:
 			BlitTemplate((Uint32*)Surface->pixels, Surface, Bitmap, PosX, PosY, x0, x1, y0, y1, Param);
 			break;
+		default:
+			PG_LogWRN("Unable to draw font: unsupported bit depth!");
+			break;
 	}
 
 	return true;
@@ -418,7 +456,7 @@ bool PG_FontEngine::BlitFTBitmap(SDL_Surface *Surface, FT_Bitmap *Bitmap, int Po
 							//if (Param->Alpha != 255)
 							//	a = (a * Param->Alpha) / 255;
 
-							raw_pixels[int(ioffset) + x + (my_charSurface->pitch/4)*(y)] = SDL_MapRGBA(my_charSurface->format, Param->Color.r, Param->Color.g, Param->Color.b, a);
+							raw_pixels[int(ioffset) + x + (my_charSurface->pitch/4)*(y)] = Param->Color.MapRGBA(my_charSurface->format, a);
 						}
 						SrcPix -= x;
 					}
@@ -436,7 +474,7 @@ bool PG_FontEngine::BlitFTBitmap(SDL_Surface *Surface, FT_Bitmap *Bitmap, int Po
 						if (Param->Alpha != 255)
 							a = (a * Param->Alpha) / 255;
 
-						*raw_pixels = SDL_MapRGBA(my_charSurface->format, Param->Color.r, Param->Color.g, Param->Color.b, a);
+						*raw_pixels = Param->Color.MapRGBA(my_charSurface->format, a);
 						raw_pixels++;
 					}
 					SrcPix -= x;
@@ -486,21 +524,36 @@ bool PG_FontEngine::RenderText(SDL_Surface *Surface, PG_Rect *ClipRect, int Base
 	FT_Face Face = FaceCache->Face;
 	FT_Vector  delta;
 
+	if(SDL_MUSTLOCK(Surface)) {
+		SDL_LockSurface(Surface);
+	}		
+	
+	PG_String sText(Text);
+
+#ifndef ENABLE_UNICODE
+	unsigned char c;
+#else
+	YChar c;
+#endif
+
 	//Go thu text and draw characters
-	for (unsigned char *c = (unsigned char*) Text;*c != 0; c++) {
+	for (PG_String::iterator c0 = sText.begin(); c0 != sText.end(); c0++) {
 		int glyph_index;
 		PG_GlyphCacheItem* Glyph;
 		int OldBaseLineX = BaseLineX;
 
-		if (*c == 0)
-			break;
+#ifndef ENABLE_UNICODE
+		c = (unsigned char)(*c0);
+#else
+		c = (*c0);
+#endif
 
 		//Skip drawing we go non-printable char
-		if (*c<32)
+		if (c<32)
 			continue;
 
 		//Get glyph index
-		glyph_index = FT_Get_Char_Index(Face, *c);
+		glyph_index = FT_Get_Char_Index(Face, c);
 
 		//Make space between characters == kerneling
 		if ( FaceCache->Use_Kerning && previous && glyph_index ) {
@@ -513,18 +566,18 @@ bool PG_FontEngine::RenderText(SDL_Surface *Surface, PG_Rect *ClipRect, int Base
 		Glyph = GetGlyph(font, glyph_index);
 
 		//Blit glyph bitmap into the surface
-		if (*c != ' ') {
+		if (c != ' ') {
 			BlitFTBitmap(Surface, &Glyph->Bitmap, BaseLineX + Glyph->Bitmap_left, BaseLineY - Glyph->Bitmap_top, font, ClipRect);
 		}
 
 		BaseLineX += Glyph->Advance_x;
-		if (font->GetStyle() & PG_FSTYLE_BOLD) {
+		if (font->GetStyle() & PG_Font::BOLD) {
 			BaseLineX += FaceCache->Bold_Offset;
 		}
 
 		//UNDERLINE
 		//TO-DO : Underline is not transparent !!!! (Fill must be replaced by Blit)
-		if (font->GetStyle() & PG_FSTYLE_UNDERLINE) {
+		if (font->GetStyle() & PG_Font::UNDERLINE) {
 			SDL_Rect	und_rect;
 
 			und_rect.x = OldBaseLineX;
@@ -535,23 +588,22 @@ bool PG_FontEngine::RenderText(SDL_Surface *Surface, PG_Rect *ClipRect, int Base
 			SDL_FillRect(
 				Surface,
 				&und_rect,
-				SDL_MapRGB(
-					Surface->format,
-					font->GetColor().r,
-					font->GetColor().g,
-					font->GetColor().b
-					)
+				font->GetColor().MapRGB(Surface->format)
 				);
 		}
 	}
 
 	//BOLD
-	if (font->GetStyle() & PG_FSTYLE_BOLD && !bRecursion) {
+	if (font->GetStyle() & PG_Font::BOLD && !bRecursion) {
 		bRecursion = true;
 		RenderText(Surface, ClipRect, OriBaseX+1, BaseLineY, Text, font);
 		bRecursion = false;
 	}
 
+	if(SDL_MUSTLOCK(Surface)) {
+		SDL_UnlockSurface(Surface);
+	}		
+	
 	return true;
 }
 
@@ -587,20 +639,32 @@ bool PG_FontEngine::GetTextSize(const char *Text, PG_Font* font, Uint16 *Width,
 			preDescent = FaceCache->Descent;
 	}
 
+	PG_String sText(Text);
+
 	//Go thu text and get sizes of the characters
-	for (unsigned char *c = (unsigned char*) Text;*c != 0; c++) {
+
+#ifndef ENABLE_UNICODE
+	unsigned char c;
+#else
+	YChar c;
+#endif
+
+	for (PG_String::iterator c0 = sText.begin(); c0 !=sText.end(); c0++) {
 		int glyph_index;
 		PG_GlyphCacheItem	*Glyph;
 
-		if (*c == 0)
-			break;
+#ifndef ENABLE_UNICODE
+		c = (unsigned char)(*c0);
+#else
+		c = (*c0);
+#endif
 
 		//Skip non-printable char
-		if (*c<32)
+		if (c<32)
 			continue;
 
 		//Get glyph index
-		glyph_index = FT_Get_Char_Index(Face, *c);
+		glyph_index = FT_Get_Char_Index(Face, c);
 
 		//Make space between characters == kerneling
 		if ( FaceCache->Use_Kerning && previous && glyph_index ) {
@@ -623,7 +687,7 @@ bool PG_FontEngine::GetTextSize(const char *Text, PG_Font* font, Uint16 *Width,
 		}
 
 		BaseLineX += Glyph->Advance_x;
-		if (font->GetStyle() & PG_FSTYLE_BOLD) {
+		if (font->GetStyle() & PG_Font::BOLD) {
 			BaseLineX += FaceCache->Bold_Offset;
 		}
 
