@@ -24,6 +24,7 @@
 #include "pglog.h"
 #include "pgpopupmenu.h"
 #include "pgmenubar.h"
+#include "pgscrollwidget.h"
 
 #include <cstring>
 #include <stdarg.h>
@@ -121,7 +122,7 @@ void PG_Layout::GetParamRect(const char **Source, char *What, PG_Rect& Rect, PG_
 	if(c == NULL) {
 		return;
 	}
-	
+
 	if(strlen(c) == 0) {
 		return;
 	}
@@ -140,8 +141,7 @@ void PG_Layout::GetParamRect(const char **Source, char *What, PG_Rect& Rect, PG_
 	for(d = strtok(parm,","); d != NULL; d = strtok(NULL,",")) {
 		if(parent == NULL) {
 			mx = ((i%2)==0) ? screen->w : screen->h;
-		}
-		else {
+		} else {
 			mx = ((i%2)==0) ? parent->w : parent->h;
 		}
 
@@ -167,23 +167,37 @@ int PG_Layout::GetParamInt(const char **Source, char *What) {
 	return atoi(p);
 }
 
-int PG_Layout::GetParamAlign(const char **Source, char *What) {
-	int	ret = -1;
+PG_ScrollBar::ScrollDirection PG_Layout::GetParamScrollDirection(const char **Source, char *What) {
+	char* p = PG_Layout::GetParamStr(Source, What);
+	PG_ScrollBar::ScrollDirection r = PG_ScrollBar::VERTICAL;
+
+	switch(atoi(p)) {
+		case 0:
+			r = PG_ScrollBar::VERTICAL;
+			break;
+			
+		case 1:
+			r = PG_ScrollBar::HORIZONTAL;
+			break;
+	}
+
+	return r;
+}
+PG_Label::TextAlign PG_Layout::GetParamAlign(const char **Source, char *What) {
+	PG_Label::TextAlign ret = PG_Label::LEFT;
 	char *c = PG_Layout::GetParamStr(Source,What);
 
 	if (c[0]==0)
-		return(-1);
+		return PG_Label::LEFT;
 
 	if (tcscmp(T(c),T("left")) == 0)
-		ret = PG_TA_LEFT;
+		ret = PG_Label::LEFT;
 	if (tcscmp(T(c),T("right")) == 0)
-		ret = PG_TA_RIGHT;
+		ret = PG_Label::RIGHT;
 	if (tcscmp(T(c),T("center")) == 0)
-		ret = PG_TA_CENTER;
+		ret = PG_Label::CENTER;
 
-	if (ret == -1)
-		PG_LogWRN("Unknown align type %s !",c);
-	return(ret);
+	return ret;
 }
 
 int PG_Layout::GetParamIMode(const char **Source, char *What) {
@@ -194,6 +208,14 @@ int PG_Layout::GetParamIMode(const char **Source, char *What) {
 		ret = BKMODE_TILE;
 	if (tcscmp(T(c),T("stretch")) == 0)
 		ret = BKMODE_STRETCH;
+	if (tcscmp(T(c),T("stretch")) == 0)
+		ret = BKMODE_STRETCH;
+	if (tcscmp(T(c),T("3tileh")) == 0)
+		ret = BKMODE_3TILEH;
+	if (tcscmp(T(c),T("3tilev")) == 0)
+		ret = BKMODE_3TILEV;
+	if (tcscmp(T(c),T("9tile")) == 0)
+		ret = BKMODE_9TILE;
 
 	return(ret);
 }
@@ -272,7 +294,7 @@ static int SetWidgetAtts(PG_Widget *Widget, const char **atts, ParseUserData_t *
 		int	r,g,b;
 
 		sscanf(c,"%d,%d,%d",&r,&g,&b);
-		Widget->SetFontColor(r,g,b);
+		Widget->SetFontColor(PG_Color(r,g,b));
 	}
 
 	c = PG_Layout::GetParamStr(atts, "fname");
@@ -287,7 +309,7 @@ static int SetWidgetAtts(PG_Widget *Widget, const char **atts, ParseUserData_t *
 
 	i = PG_Layout::GetParamInt(atts, "fstyle");
 	if (i != -1) {
-		Widget->SetFontStyle(i);
+		Widget->SetFontStyle((PG_Font::Style)i);
 	}
 
 	i = PG_Layout::GetParamInt(atts, "findex");
@@ -374,13 +396,12 @@ static int SetThemeWidgetAtts(PG_ThemeWidget *Widget, const char **atts, ParseUs
 }
 
 static int SetLabelAtts(PG_Label *Widget, const char **atts, ParseUserData_t *XMLParser) {
-	int		i;
-	char	*c;
+	PG_Label::TextAlign a;
+	int i;
+	char* c;
 
-	i = PG_Layout::GetParamAlign(atts, "align");
-	if (i != -1) {
-		Widget->SetAlignment(i);
-	}
+	a = PG_Layout::GetParamAlign(atts, "align");
+	Widget->SetAlignment(a);
 
 	i = PG_Layout::GetParamInt(atts, "indent");
 	if (i != -1) {
@@ -396,22 +417,27 @@ static int SetLabelAtts(PG_Label *Widget, const char **atts, ParseUserData_t *XM
 }
 
 static int SetUserButtonAtts(PG_Button *Widget, const char **atts, ParseUserData_t *XMLParser) {
-	char	*c,*c1;
+	char	*c,*c1,*c2;
 	int i,j,k;
 	int b;
 
 	c = PG_Layout::GetParamStr(atts, "upimage");
 	c1 = PG_Layout::GetParamStr(atts, "downimage");
+	c2 = PG_Layout::GetParamStr(atts, "overimage");
 	if (c1[0] == 0) {
 		c1 = NULL;
 	}
 
+	if (c2[0] == 0) {
+		c2 = NULL;
+	}
+
 	if (c[0] != 0) {
 		b = PG_Layout::GetParamInt(atts, "colorkey");
 		if(b != -1) {
-			Widget->SetIcon(c, c1, b);
+			Widget->SetIcon(c, c1, c2, b);
 		} else {
-			Widget->SetIcon(c, c1);
+			Widget->SetIcon(c, c1, c2);
 		}
 	}
 
@@ -421,6 +447,12 @@ static int SetUserButtonAtts(PG_Button *Widget, const char **atts, ParseUserData
 	i = PG_Layout::GetParamInt(atts, "pressed");
 	Widget->SetPressed(i == 1);
 
+	c = PG_Layout::GetParamStr(atts, "shift");
+	if (c[0] != 0) {
+		sscanf(c,"%d",&i);
+		Widget->SetShift(i);
+	}
+
 	c = PG_Layout::GetParamStr(atts, "border");
 	if (c[0] != 0) {
 		sscanf(c,"%d,%d,%d",&i,&j,&k);
@@ -440,11 +472,11 @@ static int SetButtonAtts(PG_Button *Widget, const char **atts, ParseUserData_t *
 	PG_Gradient grad;
 
 	if (PG_Layout::GetParamGrad(atts, "upgrad", &grad) != 0)
-		Widget->SetGradient(0,grad);
+		Widget->SetGradient(PG_Button::UNPRESSED, grad);
 	if (PG_Layout::GetParamGrad(atts, "downgrad", &grad) != 0)
-		Widget->SetGradient(1,grad);
+		Widget->SetGradient(PG_Button::PRESSED, grad);
 	if (PG_Layout::GetParamGrad(atts, "selgrad", &grad) != 0)
-		Widget->SetGradient(2,grad);
+		Widget->SetGradient(PG_Button::HIGHLITED, grad);
 
 	/*SetBackground*/
 
@@ -491,7 +523,7 @@ static int SetWindowAtts(PG_Window *Widget, const char **atts, ParseUserData_t *
 	if (c[0] != 0) {
 		int r,g,b;
 		sscanf(c,"%d,%d,%d",&r,&g,&b);
-		Widget->SetColorTitlebar(((b & 0xff) << 16) | ((g & 0xff) << 8) | (r & 0xFF));
+		Widget->SetTitlebarColor(((b & 0xff) << 16) | ((g & 0xff) << 8) | (r & 0xFF));
 	}
 
 	return SetThemeWidgetAtts(Widget, atts, XMLParser);
@@ -543,6 +575,14 @@ static int SetWidgetListAtts(PG_WidgetList *Widget, const char **atts, ParseUser
 	return (INHTAGFLAG_ADDWIDGET | SetThemeWidgetAtts(Widget, atts, XMLParser));
 }
 
+static int SetScrollWidgetAtts(PG_ScrollWidget*Widget, const char **atts, ParseUserData_t *XMLParser) {
+	if (PG_Layout::GetParamInt(atts, "scrollbar") == 1)
+		Widget->EnableScrollBar(1);
+
+	XMLParser->EndTagFlags |= ENDTAGFLAG_WIDGETLIST;
+	return (INHTAGFLAG_ADDWIDGET | SetThemeWidgetAtts(Widget, atts, XMLParser));
+}
+
 static int SetListBoxAtts(PG_ListBox *Widget, const char **atts, ParseUserData_t *XMLParser) {
 	Widget->SetMultiSelect(PG_Layout::GetParamInt(atts, "multisel") == 1);
 
@@ -572,10 +612,10 @@ static int SetScrollBarAtts(PG_ScrollBar *Widget, const char **atts, ParseUserDa
 		Widget->SetPosition(i);
 	}
 
-	i = PG_Layout::GetParamInt(atts, "wsize");
+	/*i = PG_Layout::GetParamInt(atts, "wsize");
 	if (i != -1) {
 		Widget->SetWindowSize(i);
-	}
+	}*/
 
 	i = PG_Layout::GetParamInt(atts, "lsize");
 	if (i != -1) {
@@ -601,7 +641,7 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 	ParseUserData_t	*XMLParser = (ParseUserData_t	*)userData;
 
 	SaveUserData(XMLParser);
-	PG_Widget* parent = ((XMLParser->EndTagFlags & INHTAGFLAG_ADDWIDGET) == INHTAGFLAG_ADDWIDGET) ? NULL : XMLParser->ParentObject;
+	PG_Widget* parent = /*((XMLParser->EndTagFlags & INHTAGFLAG_ADDWIDGET) == INHTAGFLAG_ADDWIDGET) ? NULL :*/ XMLParser->ParentObject;
 
 	//Tag <layout> <LA>
 	if (IsTag("layout","LA",XML_SECTION_DOC)) {
@@ -643,7 +683,7 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 			return;
 		}
 
-		PG_DropDown *Widget = new  PG_DropDown (parent, 0, Rect);
+		PG_DropDown *Widget = new PG_DropDown (parent, Rect);
 		XMLParser->ParentObject = Widget;
 
 		XMLParser->InhTagFlags |=SetDropDownAtts(Widget, atts, XMLParser);
@@ -663,11 +703,13 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		XMLParser->Section = XML_SECTION_BUTTON | XML_SECTION_BODY | XML_SECTION_COMWIDPARAMS;
 		PG_Layout::GetParamRect(atts, "pos", Rect, parent);
 
+		PG_LogDBG("layout: parent = %p", parent);
 		PG_Button	*Widget = new PG_Button(
-			parent,
-			0,
-			Rect,
-			PG_Layout::GetParamStr(atts, "text"));
+		                        parent,
+		                        Rect,
+		                        PG_Layout::GetParamStr(atts, "text"),
+								PG_Layout::GetParamInt(atts, "id"),
+								PG_Layout::GetParamStr(atts, "style"));
 
 		XMLParser->ParentObject = Widget;
 
@@ -680,7 +722,7 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		XMLParser->Section = XML_SECTION_LABEL | XML_SECTION_BODY | XML_SECTION_COMWIDPARAMS;
 		PG_Layout::GetParamRect(atts, "pos", Rect, parent);
 
-		PG_Label	*Widget = new PG_Label(parent, Rect, "");
+		PG_Label	*Widget = new PG_Label(parent, Rect, "", PG_Layout::GetParamStr(atts, "style"));
 		XMLParser->ParentObject = Widget;
 
 		XMLParser->InhTagFlags |=SetLabelAtts(Widget, atts, XMLParser);
@@ -772,7 +814,7 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 			flags = 0;
 		}
 
-		PG_Window *Widget = new PG_Window(parent, Rect, PG_Layout::GetParamStr(atts, "title"),flags, "Window", i);
+		PG_Window *Widget = new PG_Window(parent, Rect, PG_Layout::GetParamStr(atts, "title"), (PG_Window::WindowFlags)flags, "Window", i);
 		XMLParser->ParentObject = Widget;
 
 		XMLParser->InhTagFlags |=SetWindowAtts(Widget, atts, XMLParser);
@@ -818,18 +860,6 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		return;
 	}
 
-	//Tag <widgetlistex> <WX>
-	if (IsTag("widgetlistex","WX",XML_SECTION_BODY)) {
-		XMLParser->Section = XML_SECTION_WIDGETLISTEX | XML_SECTION_BODY | XML_SECTION_COMWIDPARAMS;
-		PG_Layout::GetParamRect(atts, "pos", Rect, parent);
-
-		PG_WidgetListEx *Widget = new PG_WidgetListEx(parent, Rect);
-		XMLParser->ParentObject = Widget;
-
-		XMLParser->InhTagFlags |= SetWidgetListAtts(Widget, atts, XMLParser);
-		return;
-	}
-
 	//Tag <listbox> <LB>
 	if (IsTag("listbox","LB",XML_SECTION_BODY)) {
 		XMLParser->Section = XML_SECTION_LISTBOX | XML_SECTION_BODY | XML_SECTION_COMWIDPARAMS;
@@ -847,12 +877,13 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		XMLParser->Section = XML_SECTION_LISTBOXITEM | XML_SECTION_COMWIDPARAMS;
 
 		int h = PG_Layout::GetParamInt(atts, "height");
-		if(h == -1) {
+		if(h < 0) {
 			h = 25;
 		}
 
-		PG_ListBoxItem *Widget = new PG_ListBoxItem(h, PG_Layout::GetParamStr(atts, "ltext"));
-		((PG_ListBox *)XMLParser->ParentObject)->AddItem(Widget);
+		// to be updated!
+		PG_ListBoxItem *Widget = new PG_ListBoxItem(parent, h, PG_Layout::GetParamStr(atts, "ltext"));
+		//((PG_ListBox *)XMLParser->ParentObject)->AddItem(Widget);
 		XMLParser->ParentObject = Widget;
 
 		XMLParser->InhTagFlags |= SetListBoxItemAtts(Widget, atts, XMLParser);
@@ -863,8 +894,9 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 	if (IsTag("columnitem","CI",XML_SECTION_LISTBOX)) {
 		XMLParser->Section = XML_SECTION_COLUMNITEM | XML_SECTION_COMWIDPARAMS;
 
-		PG_ColumnItem *Widget = new PG_ColumnItem(PG_Layout::GetParamInt(atts, "columns"),PG_Layout::GetParamInt(atts, "height"));
-		((PG_ListBox *)XMLParser->ParentObject)->AddItem(Widget);
+		// to be updated!
+		PG_ColumnItem *Widget = new PG_ColumnItem(parent, PG_Layout::GetParamInt(atts, "columns"),PG_Layout::GetParamInt(atts, "height"));
+		//((PG_ListBox *)XMLParser->ParentObject)->AddItem(Widget);
 		XMLParser->ParentObject = Widget;
 
 		XMLParser->InhTagFlags |= SetListBoxItemAtts(Widget, atts, XMLParser);
@@ -902,11 +934,11 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		PG_Layout::GetParamRect(atts, "pos", Rect, parent);
 
 		PG_RadioButton *Widget = new PG_RadioButton(
-			parent,
-			0,
-			Rect,
-			(const char*)PG_Layout::GetParamStr(atts, "text"),
-			(PG_RadioButton *)PG_Application::GetWidgetByName(PG_Layout::GetParamStr(atts, "group")));
+		                             parent,
+		                             Rect,
+		                             (const char*)PG_Layout::GetParamStr(atts, "text"),
+		                             (PG_RadioButton *)PG_Application::GetWidgetByName(PG_Layout::GetParamStr(atts, "group")),
+									 0);
 
 		XMLParser->ParentObject = Widget;
 
@@ -920,10 +952,10 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		PG_Layout::GetParamRect(atts, "pos", Rect, parent);
 
 		PG_CheckButton *Widget = new PG_CheckButton(
-			parent,
-			0,
-			Rect,
-			PG_Layout::GetParamStr(atts, "text"));
+		                             parent,
+		                             Rect,
+		                             PG_Layout::GetParamStr(atts, "text"),
+									 0);
 
 		XMLParser->ParentObject = Widget;
 
@@ -936,10 +968,10 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		XMLParser->Section = XML_SECTION_SCROLLBAR | XML_SECTION_BODY | XML_SECTION_COMWIDPARAMS;
 		PG_Layout::GetParamRect(atts, "pos", Rect, parent);
 
-		int d = PG_Layout::GetParamInt(atts, "dir");
-		(d <= 0) ? d = PG_SB_HORIZONTAL : d = PG_SB_VERTICAL;
+		PG_ScrollBar::ScrollDirection d = PG_Layout::GetParamScrollDirection(atts, "dir");
+		(d <= 0) ? d = PG_ScrollBar::HORIZONTAL : d = PG_ScrollBar::VERTICAL;
 
-		PG_ScrollBar *Widget = new PG_ScrollBar(parent, 0, Rect, d);
+		PG_ScrollBar *Widget = new PG_ScrollBar(parent, Rect, d);
 		XMLParser->ParentObject = Widget;
 
 		XMLParser->InhTagFlags |= SetScrollBarAtts(Widget, atts, XMLParser);
@@ -951,7 +983,7 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		XMLParser->Section = XML_SECTION_SCROLLBAR | XML_SECTION_BODY | XML_SECTION_COMWIDPARAMS;
 		PG_Layout::GetParamRect(atts, "pos", Rect, parent);
 
-		PG_Slider *Widget = new PG_Slider(parent, 0, Rect, PG_Layout::GetParamInt(atts, "dir")+1);
+		PG_Slider *Widget = new PG_Slider(parent, Rect, PG_Layout::GetParamScrollDirection(atts, "dir"));
 		XMLParser->ParentObject = Widget;
 
 		XMLParser->InhTagFlags |=SetScrollBarAtts(Widget, atts, XMLParser);
@@ -984,7 +1016,7 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		PG_RichEdit *Widget = new PG_RichEdit(parent, Rect, PG_Layout::GetParamInt(atts, "vresize") == 1 , PG_Layout::GetParamInt(atts, "linewidth"));
 		XMLParser->ParentObject = Widget;
 
-		XMLParser->InhTagFlags |= SetWidgetListAtts(Widget, atts, XMLParser);
+		XMLParser->InhTagFlags |= SetScrollWidgetAtts(Widget, atts, XMLParser);
 		return;
 	}
 
@@ -997,32 +1029,31 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 
 		Point.x=Rect.x;
 		Point.y=Rect.y;
-		
+
 		PG_PopupMenu* Widget = NULL;
-		
+
 		// is the current popup a child of another popup ?
 		if(XMLParser->Section & XML_SECTION_POPUPMENU) {
 			Widget = new PG_PopupMenu(NULL, 0, 0, NULL);
 			PG_PopupMenu* menu = static_cast<PG_PopupMenu*>(XMLParser->ParentObject);
 			menu->addMenuItem(PG_Layout::GetParamStr(atts, "caption"), Widget);
-		}
-		else {
+		} else {
 			// is the popup a child of a menubar ?
 			if(XMLParser->Section & XML_SECTION_MENUBAR) {
 				Widget = new PG_PopupMenu(NULL, Point.x, Point.y, PG_Layout::GetParamStr(atts, "caption"));
 				static_cast<PG_MenuBar*>(XMLParser->ParentObject)->Add(PG_Layout::GetParamStr(atts, "text"), Widget);
-			}	
-			else {
+			} else {
 				Widget = new PG_PopupMenu(parent, Point.x, Point.y, PG_Layout::GetParamStr(atts, "caption"));
 			}
 		}
-		
+
 		// register section popupmenu
 		XMLParser->Section = XML_SECTION_POPUPMENU | XML_SECTION_BODY | XML_SECTION_COMWIDPARAMS;
 
 		XMLParser->ParentObject = Widget;
 
 		XMLParser->InhTagFlags |= SetPopupMenuAtts(Widget, atts, XMLParser);
+		XMLParser->InhTagFlags |= INHTAGFLAG_HIDE; // Popups shouldn't be displayed immediately
 		return;
 	}
 
@@ -1034,10 +1065,10 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		}
 		PG_PopupMenu* menu = static_cast<PG_PopupMenu*>(XMLParser->ParentObject);
 		menu->addMenuItem(PG_Layout::GetParamStr(atts, "caption"), PG_Layout::GetParamInt(atts, "id"));
-		
+
 		return;
 	}
-	
+
 	//Tag <menubar> <MB>
 	if(IsTag("menubar", "MB", XML_SECTION_BODY)) {
 		PG_Rect Rect;
@@ -1078,6 +1109,12 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		return;
 	}
 
+	if(XMLParser->Section & XML_SECTION_HEAD) {
+		// create tag description
+		PG_XMLTag* n = new PG_XMLTag(name, atts);
+		PG_Application::GetApp()->sigXMLTag(n);
+	}
+	
 	PG_LogWRN("Unknown tag `%s` in section %d !",name,XMLParser->Section);
 	return;
 }
@@ -1095,26 +1132,30 @@ static void XMLEndDoc(void *userData, const char *name) {
 
 	if ((XMLParser->EndTagFlags & ENDTAGFLAG_OBJECT) != 0) {
 		if (((XMLParser->InhTagFlags & INHTAGFLAG_ADDWIDGET) != 0) && ((XMLParser->EndTagFlags & ENDTAGFLAG_WIDGETLIST) == 0)) {
-			WidgetToAdd = XMLParser->ParentObject;
-			goto object_end;
+			//WidgetToAdd = XMLParser->ParentObject;
+//			goto object_end;
 		}
 
-		if ((XMLParser->InhTagFlags & INHTAGFLAG_HIDE) == 0)
-			XMLParser->ParentObject->Show();
-		else
+		if ((XMLParser->InhTagFlags & INHTAGFLAG_HIDE) == 0) {
+			if(XMLParser->ParentObject->GetParent() == NULL) {
+				XMLParser->ParentObject->Show();
+			}
+		}
+		else {
 			XMLParser->ParentObject->Hide();
+		}
 
-object_end:
-		XMLParser->ParentObject->AddText("", true);
+//object_end:
+//		XMLParser->ParentObject->AddText("", true);
 	}
 
 	RestoreUserData(XMLParser);
-	
+
 	if (WidgetToAdd == NULL) {
 		return;
 	}
-	
-	((PG_WidgetList *)(XMLParser->ParentObject))->AddWidget(WidgetToAdd);
+
+	((PG_WidgetList *)(XMLParser->ParentObject))->AddChild(WidgetToAdd);
 }
 
 static void XMLTextDoc(void *userData, const XML_Char *s, int len) {
@@ -1186,15 +1227,15 @@ bool PG_Layout::Load(PG_Widget* parent, const char *filename, void (* WorkCallba
 			break;
 		}
 
-                if (bytes_read == 0) {
-                        status = true;
-                        break;
-                }
-        }
+		if (bytes_read == 0) {
+			status = true;
+			break;
+		}
+	}
 
-        if (XMLParser.Parser != NULL) {
-                XML_ParserFree(XMLParser.Parser);
-        }
+	if (XMLParser.Parser != NULL) {
+		XML_ParserFree(XMLParser.Parser);
+	}
 
 	delete f;
 
@@ -1203,3 +1244,51 @@ bool PG_Layout::Load(PG_Widget* parent, const char *filename, void (* WorkCallba
 
 	return true;
 }
+
+PG_XMLTag::PG_XMLTag(const char* n, const char** a) {
+	// fill in name
+	name = strdup(name);
+	
+	if(a == NULL) {
+		atts = NULL;
+		return;
+	}
+	
+	// get size of arry
+	const char** a0 = a;
+	int s=0;
+	while((*a0) != NULL) {
+		a0++;
+		s++;
+	}
+	
+	// alloc array of pointers
+	atts = (const char**)malloc((s+1)*sizeof(char*));
+	a0 = atts;
+	
+	// copy strings to new array
+	while(*a != NULL) {
+		*a0 = strdup(*a);
+		a0++;
+		a++;
+	}
+	*a0 = NULL;
+}
+
+PG_XMLTag::~PG_XMLTag() {
+	if(name != NULL) {
+		free((void*)name);
+	}
+	
+	if(atts == NULL) {
+		return;
+	}
+	
+	const char** a = atts;
+	while(*a) {
+		free((void*)*a);
+		a++;
+	}
+	
+	free((void*)atts);
+}
