@@ -19,23 +19,22 @@
    Alexander Pipelka
    pipelka@teleweb.at
  
-   Last Update:      $Author: pipelka $
-   Update Date:      $Date: 2002/02/09 13:45:12 $
-   Source File:      $Source: /usr/local/tigris/data/helm/cvs/repository/paragui/paragui/src/widgets/pgpopupmenu.cpp,v $
-   CVS/RCS Revision: $Revision: 1.1.4.1 $
+   Last Update:      $Author: braindead $
+   Update Date:      $Date: 2004/03/13 13:45:46 $
+   Source File:      $Source: /cvsroot/paragui/paragui/src/widgets/pgpopupmenu.cpp,v $
+   CVS/RCS Revision: $Revision: 1.3.6.4.2.7 $
    Status:           $State: Exp $
  */
 
-#include <functional>
-#include <algorithm>
-#include <iostream>
-
 #include "pgdraw.h"
 #include "pgapplication.h"
 #include "pgpopupmenu.h"
 #include "pglog.h"
+#include "pgtheme.h"
 
-using namespace std;
+#include <functional>
+#include <algorithm>
+#include <iostream>
 
 /***********************************
  * MenuItem
@@ -132,14 +131,14 @@ bool PG_PopupMenu::MenuItem::isPointInside(int x, int y) {
 	int posx = x - my_xpos;
 	int posy = y - my_ypos;
 
-	if ((posx >= 0) && (posx <= my_height) &&
-	        (posy >= 0) && (posy <= my_width))
+	if ((posx >= 0) && (posx <= my_width) &&
+	        (posy >= 0) && (posy <= my_height))
 		return true;
 
 	return false;
 }
 
-bool PG_PopupMenu::MenuItem::renderSurface(SDL_Surface *canvas, SDL_Surface **text, SDL_Color *tcol, SDL_Color *scol) {
+bool PG_PopupMenu::MenuItem::renderSurface(SDL_Surface *canvas, SDL_Surface **text, PG_Color* tcol, PG_Color* scol) {
 	if (/*!*text ||*/ !canvas)
 		return false;
 
@@ -163,21 +162,21 @@ inline bool PG_PopupMenu::MenuItem::isValidRect() {
 	return false;
 }
 
-inline bool PG_PopupMenu::MenuItem::paintNormal(SDL_Surface *canvas, SDL_Color *tcol, SDL_Color *scol) {
+inline bool PG_PopupMenu::MenuItem::paintNormal(SDL_Surface *canvas, PG_Color* tcol, PG_Color* scol) {
 	if (!isValidRect())
 		return false;
 
 	return renderSurface(canvas, &sNormal, tcol, scol);
 }
 
-inline bool PG_PopupMenu::MenuItem::paintSelected(SDL_Surface *canvas, SDL_Color *tcol, SDL_Color *scol) {
+inline bool PG_PopupMenu::MenuItem::paintSelected(SDL_Surface *canvas, PG_Color* tcol, PG_Color* scol) {
 	if (!isValidRect())
 		return false;
 
 	return renderSurface(canvas, &sSelected, tcol, scol);
 }
 
-inline bool PG_PopupMenu::MenuItem::paintDisabled(SDL_Surface *canvas, SDL_Color *tcol, SDL_Color *scol) {
+inline bool PG_PopupMenu::MenuItem::paintDisabled(SDL_Surface *canvas, PG_Color* tcol, PG_Color* scol) {
 	if (!isValidRect())
 		return false;
 
@@ -256,39 +255,24 @@ void PG_PopupMenu::appendItem(MenuItem *item) {
 
 PG_PopupMenu& PG_PopupMenu::addMenuItem(char *caption,
                                         int ID,
-                                        MSG_CALLBACK handler,
-                                        void *data,
+                                        MenuItem::MenuItemSlot handler,
+                                        PG_Pointer data,
                                         MenuItem::MI_FLAGS flags) {
 
 	MenuItem* item = new MenuItem(this, caption, ID, flags);
 	appendItem(item);
-
-	if (handler) {
-		//action = new PG_Action(ID, handler, data);
-		item->SetEventCallback(MSG_SELECTMENUITEM, handler, data);
-	}
+	item->sigSelectMenuItem.connect(handler, data);
 
 	return *this;
 }
 
 PG_PopupMenu& PG_PopupMenu::addMenuItem(char *caption,
-                                        int ID,
-                                        MSG_CALLBACK_OBJ handler,
-                                        PG_EventObject *obj,
-                                        void *data,
-                                        MenuItem::MI_FLAGS flags) {
-	if (handler && !obj) {
-		//TODO: throw an exception here
-		return *this;
-	}
+				int ID,
+				MenuItem::MI_FLAGS flags) {
 
 	MenuItem* item = new MenuItem(this, caption, ID, flags);
 	appendItem(item);
 
-	if (handler) {
-		item->SetEventObject(MSG_SELECTMENUITEM, obj, handler, data);
-	}
-
 	return *this;
 }
 
@@ -323,13 +307,13 @@ PG_PopupMenu& PG_PopupMenu::addMenuItem(char *caption,
 
 PG_PopupMenu& PG_PopupMenu::addSeparator() {
 	// Ugly
-	return addMenuItem((char*)0, -1, NULL, 0, MenuItem::MIF_SEPARATOR);
+	return addMenuItem(NULL, -1, MenuItem::MIF_SEPARATOR);
 }
 
 void PG_PopupMenu::disableItem(int id) {
 	MII mi;
 
-	mi = std::find_if(start, stop, item_with_id(id));
+	mi = find_if(start, stop, item_with_id(id));
 	if (*mi && mi != stop)
 		((MenuItem*)*mi)->disable();
 }
@@ -337,7 +321,7 @@ void PG_PopupMenu::disableItem(int id) {
 void PG_PopupMenu::enableItem(int id) {
 	MII mi;
 
-	mi = std::find_if(start, stop, item_with_id(id));
+	mi = find_if(start, stop, item_with_id(id));
 	if (*mi && mi != stop)
 		((MenuItem*)*mi)->enable();
 }
@@ -351,10 +335,16 @@ void PG_PopupMenu::trackMenu(int x, int y) {
 	x = my_xpos;
 	y = my_ypos;
 	
-	if(my_xpos + my_width >= PG_Application::GetScreenWidth()) {
-		x = PG_Application::GetScreenWidth() - my_width;
-		MoveWidget(x, y);
+	if(x + my_width >= PG_Application::GetScreenWidth()) {
+		x = PG_Application::GetScreenWidth() - my_width;		
 	}
+
+	if(y + my_height >= PG_Application::GetScreenHeight()) {
+		y = PG_Application::GetScreenHeight() - my_height;		
+	}
+
+	if (x != my_xpos || y != my_ypos)
+		MoveWidget(x, y);
 	
 	tracking = true;
 	Show();
@@ -438,9 +428,9 @@ void PG_PopupMenu::handleClick(int x, int y) {
 		if (!selected->isSubMenu()) {
 			if (!selected->isMute()) {
 				// call item's callback
-				selected->SendMessage(NULL, MSG_SELECTMENUITEM, (unsigned int)selected->getId(), (unsigned long)selected);
+				selected->sigSelectMenuItem(selected);
 				// call general callback (PG_PopupMenu)
-				SendMessage(NULL, MSG_SELECTMENUITEM, (unsigned int)selected->getId(), (unsigned long)selected);
+				sigSelectMenuItem(selected);
 			}
 		}
 
@@ -474,7 +464,7 @@ void PG_PopupMenu::liberate() {
 }
 
 void PG_PopupMenu::eventBlit(SDL_Surface* srf, const PG_Rect& src, const PG_Rect& dst) {
-	SDL_Surface* surface = my_srfScreen;
+	SDL_Surface* surface = PG_Application::GetScreen();
 	PG_ThemeWidget::eventBlit(srf, src, dst);
 
 	if (!myCaption.empty()) {
@@ -572,7 +562,6 @@ bool PG_PopupMenu::selectItem(MenuItem *item, MII iter) {
 		sub->enslave(this);
 		sub->trackMenu(my_xpos + my_width - xPadding,
 		               selected->my_ypos + my_ypos - rect.my_height);
-		sub->Show();
 
 		return true;
 	}
@@ -698,7 +687,7 @@ bool PG_PopupMenu::eventMouseButtonUp(const SDL_MouseButtonEvent *button) {
 
 bool PG_PopupMenu::eventKeyDown(const SDL_KeyboardEvent *key) {
 	SDL_KeyboardEvent key_copy = *key; // copy key structure
-	TranslateNumpadKeys(&key_copy);
+	PG_Application::TranslateNumpadKeys(&key_copy);
 	// from now, we use key_copy which was copied or translated from key
 
 	switch (key_copy.keysym.sym) {
@@ -712,9 +701,9 @@ bool PG_PopupMenu::eventKeyDown(const SDL_KeyboardEvent *key) {
 
 				if (/*actions[selected->getId()] &&*/ !selected->isDisabled()) {
 					// call item's callback
-					selected->SendMessage(NULL, MSG_SELECTMENUITEM, (unsigned int)selected->getId(), (unsigned long)selected);
+					selected->sigSelectMenuItem(selected);
 					// call general callback (PG_PopupMenu)
-					SendMessage(NULL, MSG_SELECTMENUITEM, (unsigned int)selected->getId(), (unsigned long)selected);
+					sigSelectMenuItem(selected);
 				}
 
 				selected->unselect();
@@ -775,8 +764,11 @@ void PG_PopupMenu::eventMoveWidget(int x, int y) {
 void PG_PopupMenu::eventShow() {
 	PG_ThemeWidget::eventShow();
 
-	if (tracking)
+	if (tracking) {
 		SetCapture();
+	}
+	
+	current = start;
 }
 
 void PG_PopupMenu::eventHide() {
@@ -792,48 +784,20 @@ void PG_PopupMenu::LoadThemeStyle(const char *widgettype) {
 	PG_ThemeWidget::LoadThemeStyle(widgettype);
 
 	PG_Theme *theme = PG_Application::GetTheme();
-	long      color;
 
 	// Global
-	xPadding = theme->FindProperty(widgettype, "PopupMenu", "xPadding");
-	yPadding = theme->FindProperty(widgettype, "PopupMenu", "yPadding");
+	theme->GetProperty(widgettype, "PopupMenu", "xPadding", xPadding);
+	theme->GetProperty(widgettype, "PopupMenu", "yPadding", yPadding);
 
 	// caption
 	PG_ThemeWidget::LoadThemeStyle(widgettype, "Caption");
-	color = theme->FindProperty(widgettype, "Caption", "Active");
-	captionActiveColor.r = (color >> 16) & 0xFF;
-	captionActiveColor.g = (color >> 8) & 0xFF;
-	captionActiveColor.b = color & 0xFF;
-
-	color = theme->FindProperty(widgettype, "Caption", "Inactive");
-	captionInactiveColor.r = (color >> 16) & 0xFF;
-	captionInactiveColor.g = (color >> 8) & 0xFF;
-	captionInactiveColor.b = color & 0xFF;
-
-	color = theme->FindProperty(widgettype, "MenuItem", "Normal");
-	miNormalColor.r = (color >> 16) & 0xFF;
-	miNormalColor.g = (color >> 8) & 0xFF;
-	miNormalColor.b = color & 0xFF;
-
-	color = theme->FindProperty(widgettype, "MenuItem", "Selected");
-	miSelectedColor.r = (color >> 16) & 0xFF;
-	miSelectedColor.g = (color >> 8) & 0xFF;
-	miSelectedColor.b = color & 0xFF;
-
-	color = theme->FindProperty(widgettype, "MenuItem", "Disabled");
-	miDisabledColor.r = (color >> 16) & 0xFF;
-	miDisabledColor.g = (color >> 8) & 0xFF;
-	miDisabledColor.b = color & 0xFF;
-
-	color = theme->FindProperty(widgettype, "MenuItem", "SepNormal");
-	sepNormalColor.r = (color >> 16) & 0xFF;
-	sepNormalColor.g = (color >> 8) & 0xFF;
-	sepNormalColor.b = color & 0xFF;
-
-	color = theme->FindProperty(widgettype, "MenuItem", "SepShadow");
-	sepShadowColor.r = (color >> 16) & 0xFF;
-	sepShadowColor.g = (color >> 8) & 0xFF;
-	sepShadowColor.b = color & 0xFF;
+	theme->GetColor(widgettype, "Caption", "Active", captionActiveColor);
+	theme->GetColor(widgettype, "Caption", "Inactive", captionInactiveColor);
+	theme->GetColor(widgettype, "MenuItem", "Normal", miNormalColor);
+	theme->GetColor(widgettype, "MenuItem", "Selected", miSelectedColor);
+	theme->GetColor(widgettype, "MenuItem", "Disabled", miDisabledColor);
+	theme->GetColor(widgettype, "MenuItem", "SepNormal", sepNormalColor);
+	theme->GetColor(widgettype, "MenuItem", "SepShadow", sepShadowColor);
 
 	miGradients[0] = theme->FindGradient(widgettype, "MenuItem", "gradientNormal");
 	miGradients[1] = theme->FindGradient(widgettype, "MenuItem", "gradientSelected");
@@ -843,21 +807,13 @@ void PG_PopupMenu::LoadThemeStyle(const char *widgettype) {
 	miBackgrounds[1] = theme->FindSurface(widgettype, "MenuItem", "backSelected");
 	miBackgrounds[2] = theme->FindSurface(widgettype, "MenuItem", "backDisabled");
 
-	int b;
+	theme->GetProperty(widgettype, "MenuItem", "backmodeNormal", miBkModes[0]);
+	theme->GetProperty(widgettype, "MenuItem", "backmodeSelected", miBkModes[1]);
+	theme->GetProperty(widgettype, "MenuItem", "backmodeDisabled", miBkModes[2]);
 
-	b = theme->FindProperty(widgettype, "MenuItem", "backmodeNormal");
-	miBkModes[0] = b != -1 ? b : 0;
-	b = theme->FindProperty(widgettype, "MenuItem", "backmodeSelected");
-	miBkModes[1] = b != -1 ? b : 0;
-	b = theme->FindProperty(widgettype, "MenuItem", "backmodeDisabled");
-	miBkModes[2] = b != -1 ? b : 0;
-
-	b = theme->FindProperty(widgettype, "MenuItem", "blendNormal");
-	miBlends[0] = b != -1 ? b : 0;
-	b = theme->FindProperty(widgettype, "MenuItem", "blendSelected");
-	miBlends[1] = b != -1 ? b : 0;
-	b = theme->FindProperty(widgettype, "MenuItem", "blendDisabled");
-	miBlends[2] = b != -1 ? b : 0;
+	theme->GetProperty(widgettype, "MenuItem", "blendNormal", miBlends[0]);
+	theme->GetProperty(widgettype, "MenuItem", "blendSelected", miBlends[1]);
+	theme->GetProperty(widgettype, "MenuItem", "blendDisabled", miBlends[2]);
 }
 
 void PG_PopupMenu::LoadThemeStyle(const char *widgettype, const char *objectname) {
@@ -871,3 +827,23 @@ void PG_PopupMenu::eventMouseEnter() {
 void PG_PopupMenu::eventMouseLeave() {
 	PG_ThemeWidget::eventMouseLeave();
 }
+
+bool PG_PopupMenu::SetMenuItemSlot(int id, MenuItem::MenuItemSlot slot, PG_Pointer clientdata) {
+	MII it;
+	PG_PopupMenu::MenuItem* item;
+	int itid;
+	
+	it=items.begin();
+	while(it!=items.end()) {
+		
+		itid=(*it)->getId();
+		
+		if(id==itid) {
+			item=*it;
+			item->sigSelectMenuItem.connect(slot, clientdata);
+			return true;
+		}
+		it++;
+	}
+	return false;
+}
