@@ -19,16 +19,17 @@
    Alexander Pipelka
    pipelka@teleweb.at
  
-   Last Update:      $Author: pipelka $
-   Update Date:      $Date: 2002/02/09 13:45:12 $
-   Source File:      $Source: /usr/local/tigris/data/helm/cvs/repository/paragui/paragui/src/widgets/pgrichedit.cpp,v $
-   CVS/RCS Revision: $Revision: 1.1.4.1 $
+   Last Update:      $Author: braindead $
+   Update Date:      $Date: 2004/09/05 10:51:41 $
+   Source File:      $Source: /cvsroot/paragui/paragui/src/widgets/pgrichedit.cpp,v $
+   CVS/RCS Revision: $Revision: 1.3.6.7.2.12 $
    Status:           $State: Exp $
 */
 
 
 #include "pgapplication.h"
 #include "pgrichedit.h"
+#include "pgscrollarea.h"
 #include "pglog.h"
 
 const Uint32 PG_RichEdit::my_Marks[PG_RichEdit::MARKS_COUNT] = { ' ', 0x01, '\n', '\t', 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0xF
@@ -36,107 +37,157 @@ const Uint32 PG_RichEdit::my_Marks[PG_RichEdit::MARKS_COUNT] = { ' ', 0x01, '\n'
 const Uint32 PG_RichEdit::my_FontBeginMark = 010;
 
 
-PG_RichEdit::PG_RichEdit(PG_Widget* parent, const PG_Rect& r, bool autoVerticalResize, Uint32 linewidth, Uint32 tabSize, Uint32 childsborderwidth, const char* style) :
-PG_WidgetListEx(parent, r, style) {
+PG_RichEdit::PG_RichEdit(PG_Widget* parent, const PG_Rect& r, bool autoResize, Uint32 linewidth, Uint32 tabSize, Uint32 childsborderwidth, const char* style) :
+PG_ScrollWidget(parent, r, style) {
 
-	EnableScrollBar(true, PG_SB_HORIZONTAL);
+	EnableScrollBar(true, PG_ScrollBar::HORIZONTAL);
 
-	my_listwidth = (linewidth != 0) ? linewidth : r.my_width;
+	my_LineWidth = (linewidth != 0) ? linewidth : r.my_width;
+	my_scrollarea->SetAreaWidth(my_LineWidth);
 	my_ChildsBorderWidth = childsborderwidth;
 
 	//TO-DO : Value 5 is font size, witch is currently unknown ...
 	my_objHorizontalScrollbar->SetLineSize(5);
 	my_TabSize = tabSize;
 	my_Align = my_Marks[PG_RichEdit::PG_TEXT_LEFT];
-	my_AutoVerticalResize = autoVerticalResize;
+	my_AutoVerticalResize = autoResize;
+	my_AutoHorizontalResize = autoResize;
+
+	my_objVerticalScrollbar->sigScrollTrack.connect(slot(*this, &PG_RichEdit::handleScrollTrack));
+	my_objVerticalScrollbar->sigScrollPos.connect(slot(*this, &PG_RichEdit::handleScrollTrack));
+	my_objHorizontalScrollbar->sigScrollTrack.connect(slot(*this, &PG_RichEdit::handleScrollTrack));
+	my_objHorizontalScrollbar->sigScrollPos.connect(slot(*this, &PG_RichEdit::handleScrollTrack));
 }
 
-void PG_RichEdit::UpdateScrollBarsPos() {
-	PG_WidgetList::UpdateScrollBarsPos();
+void PG_RichEdit::SetAlignment(Uint8 align) {
+	my_Align = my_Marks[align];
+}
+
+void PG_RichEdit::SetAutoResize(bool bHorizontal, bool bVertical) {
+	my_AutoHorizontalResize = bHorizontal;
+	my_AutoVerticalResize = bVertical;
+}
+
+/*void PG_RichEdit::UpdateScrollBarsPos() {
+	//PG_WidgetList::UpdateScrollBarsPos();
 
 	//TO-DO : Value 5 is font size, witch is currently unknown ...
 	my_objVerticalScrollbar->SetLineSize(5);
 	my_objHorizontalScrollbar->SetLineSize(5);
-}
+}*/
 
 void PG_RichEdit::eventBlit(SDL_Surface* srf, const PG_Rect& src, const PG_Rect& dst) {
 
-	PG_WidgetListEx::eventBlit(srf, src, dst);
+	PG_ScrollWidget::eventBlit(srf, src, dst);
 
 	RichLineArray::iterator line;
+	int deltax = (my_objHorizontalScrollbar->IsVisible()) ? my_scrollarea->GetScrollPosX() : 0;
+	int deltay = (my_objVerticalScrollbar->IsVisible()) ? my_scrollarea->GetScrollPosY() : 0;
 
 	for (line = my_RichText.begin(); line != my_RichText.end(); line++) {
-		{
-			RichLinePartArray::iterator linePart;
+		if ((Sint32)(line->my_BaseLine - deltay) < 0) {
+			continue;
+		}
 
-			for (linePart = line->my_LineParts.begin(); linePart != line->my_LineParts.end(); linePart++) {
-				Size_tArray::iterator word;
-				Uint32 width = 0;
+		RichLinePartArray::iterator linePart;
 
-				for (word = linePart->my_WordIndexes.begin(); word != linePart->my_WordIndexes.end(); word++) {
-					int deltax = (my_objHorizontalScrollbar->IsVisible()) ? my_objHorizontalScrollbar->GetPosition() : 0;
-					int deltay = (my_objVerticalScrollbar->IsVisible()) ? my_objVerticalScrollbar->GetPosition() : 0;
+		for (linePart = line->my_LineParts.begin(); linePart != line->my_LineParts.end(); linePart++) {
+			Size_tArray::iterator word;
+			Uint32 width = 0;
 
-					PG_FontEngine::RenderText(my_srfScreen, dst, my_xpos - deltax + width + linePart->my_Left,  my_ypos + line->my_BaseLine - deltay, my_ParsedWords[*word].my_Word.c_str(), GetFont());
-					width += my_ParsedWords[*word].my_WidthAfterFormating;
-				}
+			for (word = linePart->my_WordIndexes.begin(); word != linePart->my_WordIndexes.end(); word++) {
+				PG_FontEngine::RenderText(PG_Application::GetScreen(), dst, my_xpos - deltax + width + linePart->my_Left,  my_ypos + line->my_BaseLine - deltay, my_ParsedWords[*word].my_Word.c_str(), GetFont());
+				width += my_ParsedWords[*word].my_WidthAfterFormating;
 			}
 		}
-		if ((Sint32)(line->my_BaseLine - my_objVerticalScrollbar->GetPosition() - line->my_LineSpace) >= Height())
+
+		if ((Sint32)(line->my_BaseLine - deltay - line->my_LineSpace) >= Height()) {
 			break;
+		}
 	}
 }
 
-void PG_RichEdit::AddWidget(PG_Widget* w) {
-	PG_WidgetListEx::AddWidget(w);
+void PG_RichEdit::AddChild(PG_Widget* child) {
+	PG_ScrollWidget::AddChild(child);
 	CompleteLines();
 }
 
-bool PG_RichEdit::RemoveWidget(PG_Widget* w, bool shiftx, bool shifty) {
-	bool result = PG_WidgetListEx::RemoveWidget(w, shiftx, shifty);
+bool PG_RichEdit::RemoveChild(PG_Widget* child) {
+	bool result = PG_ScrollWidget::RemoveChild(child);
 	CompleteLines();
-
 	return result;
 }
 
 void PG_RichEdit::SetText(const std::string &text) {
-	my_text = text;
-	ParseWords();
-	CompleteLines();
+	SetText(text.c_str());
 }
 
 void PG_RichEdit::SetText(const char *text) {
-	PG_Widget::SetText(text);
+	if(text == NULL) {
+		my_text = "";
+		return;
+	}
+
+	my_scrollarea->SetAreaWidth(my_LineWidth);
+	my_scrollarea->SetAreaHeight(0);
+	
+	my_text = text;
+		
+	// trim the string
+    bool bStop = false;
+	char c;
+
+    while(!my_text.empty() && !bStop) {
+		c = my_text[my_text.size()-1];
+		bStop = true;
+		switch(c) {
+			case '\n':
+			case '\r':
+			case '\t':
+			case ' ':
+				my_text = my_text.substr(0, my_text.size()-1);
+				bStop = false;
+		}
+	}
+
 	ParseWords();
 	CompleteLines();
 }
 
 void PG_RichEdit::ParseWords() {
 	size_t searchFrom = 0;
-	std::string word;
 
 	my_ParsedWords.clear();
 
-	do {
+	Uint16 my_sw = 0;
+	PG_FontEngine::GetTextSize(" ", GetFont(), &my_sw);
+
 		RichWordDescription wordDescr;
+	std::string word;
+
 		Uint16 w, h, sw = 0;
 		int bl, ls;
 		bool space = false;
+	int length = 0;
+	
+	do {		
+		sw = 0;		
+		space = false;		
 
 		searchFrom = GetWord(searchFrom, &word, &wordDescr.my_EndMark);
 
-		int length = word.length();
+		length = word.length();
 		if (length > 0) {
 			if (word[length - 1] == ' ') {
 				space = true;
 				word.erase(length-1, 1);
 			}
 		}
-		PG_FontEngine::GetTextSize(word.c_str(), GetFont(), &w, &h, &bl, &ls, NULL, NULL, NULL);
+		PG_FontEngine::GetTextSize(word.c_str(), GetFont(), &w, &h, &bl, &ls);
 		wordDescr.my_Width = w;
 		if (space) {
 			word += ' ';
-			PG_FontEngine::GetTextSize(" ", GetFont(), &sw);
+			sw = my_sw;
 		}
 		wordDescr.my_Word = word;
 		wordDescr.my_EndSpaceWidth = sw;
@@ -170,6 +221,7 @@ size_t PG_RichEdit::GetWord(size_t searchFrom, std::string *word, Uint32 *endMar
 				result = ik;
 				jk = MARKS_COUNT;
 				ik = length;
+				break;
 			}
 		}
 	}
@@ -180,6 +232,21 @@ size_t PG_RichEdit::GetWord(size_t searchFrom, std::string *word, Uint32 *endMar
 		if ((Uint32)my_text[result] != my_Marks[MARK_SPACE])
 			increment = 0;
 		*word = my_text.substr(searchFrom, result - searchFrom + increment);
+		
+		// Candid - cut too long words, if wanted
+		if (!my_AutoHorizontalResize) {
+			Uint16 width, i = word->size() - 1, w;
+			PG_FontEngine::GetTextSize(word->c_str(), GetFont(), &width);
+
+			if (width > my_width) {			
+				for (; width > my_width && i > 0; --i) {				
+					PG_FontEngine::GetTextSize(word->substr(i, 1).c_str(), GetFont(), &w);
+					width -= w;
+				}
+				result -= word->size() - i + 1;
+				*word = word->substr(0, i);
+			}
+		}						
 
 		if ((Uint32)my_text[result] == my_Marks[MARK_NONBREAKABLE_SPACE]) {
 			std::string newword;
@@ -190,6 +257,19 @@ size_t PG_RichEdit::GetWord(size_t searchFrom, std::string *word, Uint32 *endMar
 		}
 	} else {
 		*word = my_text.substr(searchFrom);
+		// Candid - cut too long words, if wanted
+		if (!my_AutoHorizontalResize) {
+			Uint16 width, i = word->size() - 1, w;
+			PG_FontEngine::GetTextSize(word->c_str(), GetFont(), &width);
+
+			if (width > my_width) {			
+				for (; width > my_width && i > 0; --i) {				
+					PG_FontEngine::GetTextSize(word->substr(i, 1).c_str(), GetFont(), &w);
+					width -= w;
+				}				
+				*word = word->substr(0, i);
+			}
+		}
 	}
 
 	return result;
@@ -230,16 +310,25 @@ Sint32 PG_RichEdit::CompleteLines() {
 
 	} while(searchFrom < my_ParsedWords.size());
 
-	if (top > (Sint32)my_listheight) {
-		my_listheight = top;
+	if (top > my_scrollarea->GetAreaHeight()) {
+		my_scrollarea->SetAreaHeight(top);
 	}
 
+	if(my_AutoVerticalResize || my_AutoHorizontalResize) {
+		Uint16 w = my_width, h = my_height;
 	if (my_AutoVerticalResize) {
-		SizeWidget(my_width, my_listheight);
+			h = GetListHeight();
+		}
+		if(my_AutoHorizontalResize) {
+			w = GetListWidth();
+	}
+		SizeWidget(w, h, false);
+	}
+
+	else {
+		CheckScrollBars();
 	}
 
-	CheckScrollBars();
-	UpdateScrollBarsPos();
 	Update();
 
 	return top;
@@ -248,7 +337,8 @@ Sint32 PG_RichEdit::CompleteLines() {
 size_t PG_RichEdit::CompleteLine(RichLineArray::iterator actualLine, Sint32 &lineTop, size_t searchFrom, Uint32 &lineSpace, Uint32 &lineAscent, bool changeAlign) {
 	bool breakLine = false;
 
-	Sint32 linePartLeft = 0, linePartWidthMax = my_listwidth;
+	Sint32 linePartLeft = 0;
+	Sint32 linePartWidthMax = my_scrollarea->GetAreaWidth();
 	WidgetMap widgetsOnLine;
 
 	Uint32 align = my_Align;
@@ -288,7 +378,7 @@ size_t PG_RichEdit::CompleteLine(RichLineArray::iterator actualLine, Sint32 &lin
 	}
 
 	do {
-		linePartWidthMax = my_listwidth - linePartLeft;
+		linePartWidthMax = my_scrollarea->GetAreaWidth() - linePartLeft;
 		if (childOnLine != widgetsOnLine.end())
 			linePartWidthMax = childOnLine->first - linePartLeft;
 		RichLinePartArray::iterator actualLinePart = actualLine->my_LineParts.insert(actualLine->my_LineParts.end(), RichLinePart(linePartLeft, linePartWidthMax));
@@ -335,9 +425,9 @@ void PG_RichEdit::AlignLine(RichLineArray::iterator actualLine, WidgetMap &widge
 		}
 
 		if (align == my_Marks[MARK_ALL_CENTER])
-			delta = (my_listwidth / 2) - (lineWidth / 2);
+			delta = (my_scrollarea->GetAreaWidth() / 2) - (lineWidth / 2);
 		else if (align == my_Marks[MARK_ALL_RIGHT])
-			delta = my_listwidth - lineWidth;
+			delta = my_scrollarea->GetAreaWidth() - lineWidth;
 
 		if (align != my_Marks[MARK_ALL_LEFT]) {
 			for (linePart = actualLine->my_LineParts.begin(); linePart < actualLine->my_LineParts.end(); linePart++) {
@@ -417,11 +507,19 @@ Sint32 PG_RichEdit::CompleteLinePart(size_t searchFrom, Sint32 lineTop, Uint32 &
 			breakLine = false;
 			searchFrom = oldFind - 1;
 
-			if (actualLinePart->my_WordIndexes.size() == 0) {
+			if (w > actualLinePart->my_WidthMax) {				
+				if(my_AutoHorizontalResize) {					
+					searchFrom--;
+					linePartEnd = false;					
+					my_scrollarea->SetAreaWidth(lineWidth);					
+				}								
+			}
+
+			else if (actualLinePart->my_WordIndexes.size() == 0) {
 				//searchFrom++;
 				lineSpace = my_ParsedWords[searchFrom + 1].my_LineSkip;
 			}
-			linePartEnd = true;
+			//linePartEnd = true;
 		} else {
 			if ((my_ParsedWords[searchFrom].my_Word.length() > 0) || (tabSize > 0)) {
 				if ((Uint32)ls > lineSpace)
@@ -475,29 +573,34 @@ void PG_RichEdit::AlignLinePart(RichLinePartArray::iterator actualLinePart, Uint
 }
 
 void PG_RichEdit::GetWidgetsOnLine(Sint32 lineTop, Uint32 lineHeight, WidgetMap &widgetsOnLine, bool clear) {
-	std::vector<PG_Widget*>::iterator child;
+	PG_Widget* child;
 
 	if (clear)
 		widgetsOnLine.clear();
 
-	for (child = my_widgetList.begin(); child < my_widgetList.end(); child++) {
-		PG_Point coord = ScreenToClient((*child)->my_xpos, (*child)->my_ypos);
+	PG_RectList* list = my_scrollarea->GetChildList();
+	if(list == NULL) {
+		return;
+	}
 
-		if ((lineTop < (coord.y + (*child)->my_height)) && ((lineTop + (Sint32)lineHeight) >= coord.y)) {
+	for (child = list->first(); child != NULL; child = child->next()) {
+		PG_Point coord = ScreenToClient(child->my_xpos, child->my_ypos);
+
+		if ((lineTop < (coord.y + child->my_height)) && ((lineTop + (Sint32)lineHeight) >= coord.y)) {
 			bool insert = true;
 
 			if (!clear) {
 				WidgetMap::iterator header;
 
 				for (header = widgetsOnLine.begin(); header != widgetsOnLine.end(); header++) {
-					if (*child == header->second) {
+					if (child == header->second) {
 						insert = false;
 						break;
 					}
 				}
 			}
 			if (insert) {
-				widgetsOnLine.insert(WidgetMap::value_type(coord.x - my_ChildsBorderWidth, *child));
+				widgetsOnLine.insert(WidgetMap::value_type(coord.x - my_ChildsBorderWidth, child));
 			}
 		}
 	}
@@ -517,7 +620,7 @@ bool PG_RichEdit::ProcessLongLine(std::string &word, size_t &searchFrom, Uint32
 	    ik += CurFont.FormatTagProc(&word[ik], 0);
 	    }
 	    
-	    std::string newword = word.substr(0, ((ik == std::string::npos) ? ik : ik + 1));           
+	    string newword = word.substr(0, ((ik == string::npos) ? ik : ik + 1));           
 	    PG_FontEngine::GetTextSize(newword.c_str(), GetFont(), &w, &h, &la, &ls, NULL, NULL, NULL);
 
 	    if (w > (Sint32)my_listwidth)
@@ -551,3 +654,39 @@ bool PG_RichEdit::ProcessLongLine(std::string &word, size_t &searchFrom, Uint32
 }    */
 	return true;
 }
+
+bool PG_RichEdit::LoadText(const char* textfile) {
+	PG_DataContainer* text = PG_FileArchive::ReadFile(textfile);
+	
+	if(text == NULL) {
+		return false;
+	}
+	
+	// Hmm,...
+	// size() returns the number of loaded bytes. We have to terminate the text with 0.
+	// Last character will be truncated by this (i love valgrind).
+	text->data()[text->size()-1] = 0;
+	SetText(text->data());
+	
+	delete text;
+	return true;
+}
+
+void PG_RichEdit::SetTabSize(Uint16 tabSize) {
+	my_TabSize = tabSize;
+}
+
+bool PG_RichEdit::handleScrollTrack() {
+	my_scrollarea->Update();
+	return true;
+}
+
+void PG_RichEdit::SetLineWidth(Uint16 lineWidth) {
+	my_LineWidth = lineWidth;
+	SetText(GetText());
+}
+
+void PG_RichEdit::eventSizeWidget(Uint16 w, Uint16 h) {
+	PG_ScrollWidget::eventSizeWidget(w, h);
+	SetLineWidth(w);
+}
