@@ -1,8 +1,8 @@
 #include "paragui.h"
-#include <iostream>
 #include "pgapplication.h"
+#include "pgwindow.h"
 
-int main() {
+int main( int argc, char **argv ) {
 
 	// construct the application object
 	PG_Application app;
@@ -17,7 +17,7 @@ int main() {
 	}
 
 	app.LoadLayout("pokus.xml");
-
+	
 	// Enter main loop 
 	app.Run();
 
