@@ -10,13 +10,10 @@ int main( int argc, char **argv )
 
    app.InitScreen( 640, 480 );
 
-   PG_Window a( NULL, PG_Rect( 10, 10, 100, 100 ),
-"Window 98, the most reliable window yet." );
-   PG_Window b( NULL, PG_Rect( 60, 110, 100, 100 ),
-"Window XP, ." );
+   PG_Window a( NULL, PG_Rect( 10, 10, 100, 100 ), "Window 98, the most reliable window yet.");
+   PG_Window b( NULL, PG_Rect( 60, 110, 100, 100 ), "Window XP, .");
 
    a.SizeWidget( 400, 200 );
-
    b.SizeWidget( 400, 200 );
 
    a.Show();
